"""macgen_bin - Generate application from shared libraries"""

import os
import sys
import string
import types
import macfs
from MACFS import *
import MacOS
from Carbon import Res
import py_resource
import cfmfile
import buildtools


def generate(input, output, module_dict=None, architecture='fat', debug=0):
    # try to remove old file
    try:
        os.remove(output)
    except:
        pass

    if module_dict is None:
        import macmodulefinder
        print "Searching for modules..."
        module_dict, missing = macmodulefinder.process(input, [], [], 1)
        if missing:
            import EasyDialogs
            missing.sort()
            answer = EasyDialogs.AskYesNoCancel("Some modules could not be found; continue anyway?\n(%s)"
                            % string.join(missing, ", "))
            if answer <> 1:
                sys.exit(0)

    applettemplatepath = buildtools.findtemplate()
    corepath = findpythoncore()

    dynamicmodules, dynamicfiles, extraresfiles = findfragments(module_dict, architecture)

    print 'Adding "__main__"'
    buildtools.process(applettemplatepath, input, output, 0)

    outputref = Res.FSpOpenResFile(output, 3)
    try:
        Res.UseResFile(outputref)

        print "Adding Python modules"
        addpythonmodules(module_dict)

        print "Adding PythonCore resources"
        copyres(corepath, outputref, ['cfrg', 'Popt', 'GU\267I'], 1)

        print "Adding resources from shared libraries"
        for ppcpath, cfm68kpath in extraresfiles:
            if os.path.exists(ppcpath):
                copyres(ppcpath, outputref, ['cfrg'], 1)
            elif os.path.exists(cfm68kpath):
                copyres(cfm68kpath, outputref, ['cfrg'], 1)

        print "Fixing sys.path prefs"
        Res.UseResFile(outputref)
        try:
            res = Res.Get1Resource('STR#', 228) # from PythonCore
        except Res.Error: pass
        else:
            res.RemoveResource()
        # setting pref file name to empty string
        res = Res.Get1NamedResource('STR ', "PythonPreferenceFileName")
        res.data = Pstring("")
        res.ChangedResource()
        syspathpref = "$(APPLICATION)"
        res = Res.Resource("\000\001" + Pstring(syspathpref))
        res.AddResource("STR#", 229, "sys.path preference")

        print "Creating 'PYD ' resources"
        for modname, (ppcfrag, cfm68kfrag) in dynamicmodules.items():
            res = Res.Resource(Pstring(ppcfrag) + Pstring(cfm68kfrag))
            id = 0
            while id < 128:
                id = Res.Unique1ID('PYD ')
            res.AddResource('PYD ', id, modname)
    finally:
        Res.CloseResFile(outputref)
    print "Merging code fragments"
    cfmfile.mergecfmfiles([applettemplatepath, corepath] + dynamicfiles.keys(),
                    output, architecture)

    print "done!"


def findfragments(module_dict, architecture):
    dynamicmodules = {}
    dynamicfiles = {}
    extraresfiles = []
    for name, module in module_dict.items():
        if module.gettype() <> 'dynamic':
            continue
        path = resolvealiasfile(module.__file__)
        dir, filename = os.path.split(path)
##              ppcfile, cfm68kfile = makefilenames(filename)
        ppcfile = filename
        cfm68kfile = "dummy.cfm68k.slb"

        # ppc stuff
        ppcpath = os.path.join(dir, ppcfile)
        if architecture <> 'm68k':
            ppcfrag, dynamicfiles = getfragname(ppcpath, dynamicfiles)
        else:
            ppcfrag = "_no_fragment_"

        # 68k stuff
        cfm68kpath = os.path.join(dir, cfm68kfile)
        if architecture <> 'pwpc':
            cfm68kfrag, dynamicfiles = getfragname(cfm68kpath, dynamicfiles)
        else:
            cfm68kfrag = "_no_fragment_"

        dynamicmodules[name] = ppcfrag, cfm68kfrag
        if (ppcpath, cfm68kpath) not in extraresfiles:
            extraresfiles.append((ppcpath, cfm68kpath))
    return dynamicmodules, dynamicfiles, extraresfiles


def getfragname(path, dynamicfiles):
    if not dynamicfiles.has_key(path):
        if os.path.exists(path):
            lib = cfmfile.CfrgResource(path)
            fragname = lib.fragments[0].name
        else:
            print "shared lib not found:", path
            fragname = "_no_fragment_"
        dynamicfiles[path] = fragname
    else:
        fragname = dynamicfiles[path]
    return fragname, dynamicfiles


def addpythonmodules(module_dict):
    # XXX should really use macgen_rsrc.generate(), this does the same, but skips __main__
    items = module_dict.items()
    items.sort()
    for name, module in items:
        mtype = module.gettype()
        if mtype not in ['module', 'package'] or name == "__main__":
            continue
        location = module.__file__

        if location[-4:] == '.pyc':
            # Attempt corresponding .py
            location = location[:-1]
        if location[-3:] != '.py':
            print '*** skipping', location
            continue

        print 'Adding module "%s"' % name
        id, name = py_resource.frompyfile(location, name, preload=0,
                        ispackage=mtype=='package')

def Pstring(str):
    if len(str) > 255:
        raise TypeError, "Str255 must be at most 255 chars long"
    return chr(len(str)) + str

##def makefilenames(name):
##      lname = string.lower(name)
##      pos = string.find(lname, ".ppc.")
##      if pos > 0:
##              return name, name[:pos] + '.CFM68K.' + name[pos+5:]
##      pos = string.find(lname, ".cfm68k.")
##      if pos > 0:
##              return name[:pos] + '.ppc.' + name[pos+8:], name
##      raise ValueError, "can't make ppc/cfm68k filenames"

def copyres(input, output, *args, **kwargs):
    openedin = openedout = 0
    if type(input) == types.StringType:
        input = Res.FSpOpenResFile(input, 1)
        openedin = 1
    if type(output) == types.StringType:
        output = Res.FSpOpenResFile(output, 3)
        openedout = 1
    try:
        apply(buildtools.copyres, (input, output) + args, kwargs)
    finally:
        if openedin:
            Res.CloseResFile(input)
        if openedout:
            Res.CloseResFile(output)

def findpythoncore():
    """find the PythonCore shared library, possibly asking the user if we can't find it"""

    try:
        vRefNum, dirID = macfs.FindFolder(kOnSystemDisk, kSharedLibrariesFolderType, 0)
    except macfs.error:
        extpath = ":"
    else:
        extpath = macfs.FSSpec((vRefNum, dirID, "")).as_pathname()
    version = string.split(sys.version)[0]
    if MacOS.runtimemodel == 'carbon':
        corename = "PythonCoreCarbon " + version
    elif MacOS.runtimemodel == 'ppc':
        corename = "PythonCore " + version
    else:
        raise "Unknown MacOS.runtimemodel", MacOS.runtimemodel
    corepath = os.path.join(extpath, corename)
    if not os.path.exists(corepath):
        corepath = EasyDialogs.AskFileForOpen(message="Please locate PythonCore:",
                typeList=("shlb",))
        if not corepath:
            raise KeyboardInterrupt, "cancelled"
    return resolvealiasfile(corepath)

def resolvealiasfile(path):
    try:
        fss, dummy1, dummy2 = macfs.ResolveAliasFile(path)
    except macfs.error:
        pass
    else:
        path = fss.as_pathname()
    return path



# Copyright (c) 2004 Python Software Foundation.
# All rights reserved.

# Written by Eric Price <eprice at tjhsst.edu>
#    and Facundo Batista <facundo at taniquetil.com.ar>
#    and Raymond Hettinger <python at rcn.com>
#    and Aahz (aahz at pobox.com)
#    and Tim Peters

"""
These are the test cases for the Decimal module.

There are two groups of tests, Arithmetic and Behaviour. The former test
the Decimal arithmetic using the tests provided by Mike Cowlishaw. The latter
test the pythonic behaviour according to PEP 327.

Cowlishaw's tests can be downloaded from:

   www2.hursley.ibm.com/decimal/dectest.zip

This test module can be called from command line with one parameter (Arithmetic
or Behaviour) to test each part, or without parameter to test both parts. If
you're working through IDLE, you can import this test module and call test_main()
with the corresponding argument.
"""

from __future__ import division

import unittest
import glob
import os, sys
import pickle, copy
from decimal import *
from test.test_support import TestSkipped, run_unittest, run_doctest, is_resource_enabled
import threading
import random

# Useful Test Constant
Signals = getcontext().flags.keys()

# Tests are built around these assumed context defaults
DefaultContext.prec=9
DefaultContext.rounding=ROUND_HALF_EVEN
DefaultContext.traps=dict.fromkeys(Signals, 0)
setcontext(DefaultContext)


TESTDATADIR = 'decimaltestdata'
if __name__ == '__main__':
    file = sys.argv[0]
else:
    file = __file__
testdir = os.path.dirname(file) or os.curdir
dir = testdir + os.sep + TESTDATADIR + os.sep

skip_expected = not os.path.isdir(dir)

# Make sure it actually raises errors when not expected and caught in flags
# Slower, since it runs some things several times.
EXTENDEDERRORTEST = False


#Map the test cases' error names to the actual errors

ErrorNames = {'clamped' : Clamped,
              'conversion_syntax' : InvalidOperation,
              'division_by_zero' : DivisionByZero,
              'division_impossible' : InvalidOperation,
              'division_undefined' : InvalidOperation,
              'inexact' : Inexact,
              'invalid_context' : InvalidOperation,
              'invalid_operation' : InvalidOperation,
              'overflow' : Overflow,
              'rounded' : Rounded,
              'subnormal' : Subnormal,
              'underflow' : Underflow}


def Nonfunction(*args):
    """Doesn't do anything."""
    return None

RoundingDict = {'ceiling' : ROUND_CEILING, #Maps test-case names to roundings.
                'down' : ROUND_DOWN,
                'floor' : ROUND_FLOOR,
                'half_down' : ROUND_HALF_DOWN,
                'half_even' : ROUND_HALF_EVEN,
                'half_up' : ROUND_HALF_UP,
                'up' : ROUND_UP}

# Name adapter to be able to change the Decimal and Context
# interface without changing the test files from Cowlishaw
nameAdapter = {'toeng':'to_eng_string',
               'tosci':'to_sci_string',
               'samequantum':'same_quantum',
               'tointegral':'to_integral',
               'remaindernear':'remainder_near',
               'divideint':'divide_int',
               'squareroot':'sqrt',
               'apply':'_apply',
              }

class DecimalTest(unittest.TestCase):
    """Class which tests the Decimal class against the test cases.

    Changed for unittest.
    """
    def setUp(self):
        global dir
        self.context = Context()
        for key in DefaultContext.traps.keys():
            DefaultContext.traps[key] = 1
        self.ignore_list = ['#']
        # Basically, a # means return NaN InvalidOperation.
        # Different from a sNaN in trim

        self.ChangeDict = {'precision' : self.change_precision,
                      'rounding' : self.change_rounding_method,
                      'maxexponent' : self.change_max_exponent,
                      'minexponent' : self.change_min_exponent,
                      'clamp' : self.change_clamp}

    def tearDown(self):
        """Cleaning up enviroment."""
        # leaving context in original state
        for key in DefaultContext.traps.keys():
            DefaultContext.traps[key] = 0
        return

    def eval_file(self, file):
        global skip_expected
        if skip_expected:
            raise TestSkipped
            return
        for line in open(file).xreadlines():
            line = line.replace('\r\n', '').replace('\n', '')
            #print line
            try:
                t = self.eval_line(line)
            except InvalidOperation:
                print 'Error in test cases:'
                print line
                continue
            except DecimalException, exception:
                #Exception raised where there shoudn't have been one.
                self.fail('Exception "'+exception.__class__.__name__ + '" raised on line '+line)

        return

    def eval_line(self, s):
        if s.find(' -> ') >= 0 and s[:2] != '--' and not s.startswith('  --'):
            s = (s.split('->')[0] + '->' +
                 s.split('->')[1].split('--')[0]).strip()
        else:
            s = s.split('--')[0].strip()

        for ignore in self.ignore_list:
            if s.find(ignore) >= 0:
                #print s.split()[0], 'NotImplemented--', ignore
                return
        if not s:
            return
        elif ':' in s:
            return self.eval_directive(s)
        else:
            return self.eval_equation(s)

    def eval_directive(self, s):
        funct, value = map(lambda x: x.strip().lower(), s.split(':'))
        if funct == 'rounding':
            value = RoundingDict[value]
        else:
            try:
                value = int(value)
            except ValueError:
                pass

        funct = self.ChangeDict.get(funct, Nonfunction)
        funct(value)

    def eval_equation(self, s):
        #global DEFAULT_PRECISION
        #print DEFAULT_PRECISION
        try:
            Sides = s.split('->')
            L = Sides[0].strip().split()
            id = L[0]
#            print id,
            funct = L[1].lower()
            valstemp = L[2:]
            L = Sides[1].strip().split()
            ans = L[0]
            exceptions = L[1:]
        except (TypeError, AttributeError, IndexError):
            raise InvalidOperation
        def FixQuotes(val):
            val = val.replace("''", 'SingleQuote').replace('""', 'DoubleQuote')
            val = val.replace("'", '').replace('"', '')
            val = val.replace('SingleQuote', "'").replace('DoubleQuote', '"')
            return val
        fname = nameAdapter.get(funct, funct)
        if fname == 'rescale':
            return
        funct = getattr(self.context, fname)
        vals = []
        conglomerate = ''
        quote = 0
        theirexceptions = [ErrorNames[x.lower()] for x in exceptions]

        for exception in Signals:
            self.context.traps[exception] = 1 #Catch these bugs...
        for exception in theirexceptions:
            self.context.traps[exception] = 0
        for i, val in enumerate(valstemp):
            if val.count("'") % 2 == 1:
                quote = 1 - quote
            if quote:
                conglomerate = conglomerate + ' ' + val
                continue
            else:
                val = conglomerate + val
                conglomerate = ''
            v = FixQuotes(val)
            if fname in ('to_sci_string', 'to_eng_string'):
                if EXTENDEDERRORTEST:
                    for error in theirexceptions:
                        self.context.traps[error] = 1
                        try:
                            funct(self.context.create_decimal(v))
                        except error:
                            pass
                        except Signals, e:
                            self.fail("Raised %s in %s when %s disabled" % \
                                      (e, s, error))
                        else:
                            self.fail("Did not raise %s in %s" % (error, s))
                        self.context.traps[error] = 0
                v = self.context.create_decimal(v)
            else:
                v = Decimal(v)
            vals.append(v)

        ans = FixQuotes(ans)

        if EXTENDEDERRORTEST and fname not in ('to_sci_string', 'to_eng_string'):
            for error in theirexceptions:
                self.context.traps[error] = 1
                try:
                    funct(*vals)
                except error:
                    pass
                except Signals, e:
                    self.fail("Raised %s in %s when %s disabled" % \
                              (e, s, error))
                else:
                    self.fail("Did not raise %s in %s" % (error, s))
                self.context.traps[error] = 0
        try:
            result = str(funct(*vals))
            if fname == 'same_quantum':
                result = str(int(eval(result))) # 'True', 'False' -> '1', '0'
        except Signals, error:
            self.fail("Raised %s in %s" % (error, s))
        except: #Catch any error long enough to state the test case.
            print "ERROR:", s
            raise

        myexceptions = self.getexceptions()
        self.context.clear_flags()

        myexceptions.sort()
        theirexceptions.sort()

        self.assertEqual(result, ans,
                         'Incorrect answer for ' + s + ' -- got ' + result)
        self.assertEqual(myexceptions, theirexceptions,
                         'Incorrect flags set in ' + s + ' -- got ' \
                         + str(myexceptions))
        return

    def getexceptions(self):
        return [e for e in Signals if self.context.flags[e]]

    def change_precision(self, prec):
        self.context.prec = prec
    def change_rounding_method(self, rounding):
        self.context.rounding = rounding
    def change_min_exponent(self, exp):
        self.context.Emin = exp
    def change_max_exponent(self, exp):
        self.context.Emax = exp
    def change_clamp(self, clamp):
        self.context._clamp = clamp

# Dynamically build custom test definition for each file in the test
# directory and add the definitions to the DecimalTest class.  This
# procedure insures that new files do not get skipped.
for filename in os.listdir(dir):
    if '.decTest' not in filename:
        continue
    head, tail = filename.split('.')
    tester = lambda self, f=filename: self.eval_file(dir + f)
    setattr(DecimalTest, 'test_' + head, tester)
    del filename, head, tail, tester



# The following classes test the behaviour of Decimal according to PEP 327

class DecimalExplicitConstructionTest(unittest.TestCase):
    '''Unit tests for Explicit Construction cases of Decimal.'''

    def test_explicit_empty(self):
        self.assertEqual(Decimal(), Decimal("0"))

    def test_explicit_from_None(self):
        self.assertRaises(TypeError, Decimal, None)

    def test_explicit_from_int(self):

        #positive
        d = Decimal(45)
        self.assertEqual(str(d), '45')

        #very large positive
        d = Decimal(500000123)
        self.assertEqual(str(d), '500000123')

        #negative
        d = Decimal(-45)
        self.assertEqual(str(d), '-45')

        #zero
        d = Decimal(0)
        self.assertEqual(str(d), '0')

    def test_explicit_from_string(self):

        #empty
        self.assertEqual(str(Decimal('')), 'NaN')

        #int
        self.assertEqual(str(Decimal('45')), '45')

        #float
        self.assertEqual(str(Decimal('45.34')), '45.34')

        #engineer notation
        self.assertEqual(str(Decimal('45e2')), '4.5E+3')

        #just not a number
        self.assertEqual(str(Decimal('ugly')), 'NaN')

    def test_explicit_from_tuples(self):

        #zero
        d = Decimal( (0, (0,), 0) )
        self.assertEqual(str(d), '0')

        #int
        d = Decimal( (1, (4, 5), 0) )
        self.assertEqual(str(d), '-45')

        #float
        d = Decimal( (0, (4, 5, 3, 4), -2) )
        self.assertEqual(str(d), '45.34')

        #weird
        d = Decimal( (1, (4, 3, 4, 9, 1, 3, 5, 3, 4), -25) )
        self.assertEqual(str(d), '-4.34913534E-17')

        #wrong number of items
        self.assertRaises(ValueError, Decimal, (1, (4, 3, 4, 9, 1)) )

        #bad sign
        self.assertRaises(ValueError, Decimal, (8, (4, 3, 4, 9, 1), 2) )

        #bad exp
        self.assertRaises(ValueError, Decimal, (1, (4, 3, 4, 9, 1), 'wrong!') )

        #bad coefficients
        self.assertRaises(ValueError, Decimal, (1, (4, 3, 4, None, 1), 2) )
        self.assertRaises(ValueError, Decimal, (1, (4, -3, 4, 9, 1), 2) )

    def test_explicit_from_Decimal(self):

        #positive
        d = Decimal(45)
        e = Decimal(d)
        self.assertEqual(str(e), '45')
        self.assertNotEqual(id(d), id(e))

        #very large positive
        d = Decimal(500000123)
        e = Decimal(d)
        self.assertEqual(str(e), '500000123')
        self.assertNotEqual(id(d), id(e))

        #negative
        d = Decimal(-45)
        e = Decimal(d)
        self.assertEqual(str(e), '-45')
        self.assertNotEqual(id(d), id(e))

        #zero
        d = Decimal(0)
        e = Decimal(d)
        self.assertEqual(str(e), '0')
        self.assertNotEqual(id(d), id(e))

    def test_explicit_context_create_decimal(self):

        nc = copy.copy(getcontext())
        nc.prec = 3

        # empty
        d = Decimal()
        self.assertEqual(str(d), '0')
        d = nc.create_decimal()
        self.assertEqual(str(d), '0')

        # from None
        self.assertRaises(TypeError, nc.create_decimal, None)

        # from int
        d = nc.create_decimal(456)
        self.failUnless(isinstance(d, Decimal))
        self.assertEqual(nc.create_decimal(45678),
                         nc.create_decimal('457E+2'))

        # from string
        d = Decimal('456789')
        self.assertEqual(str(d), '456789')
        d = nc.create_decimal('456789')
        self.assertEqual(str(d), '4.57E+5')

        # from tuples
        d = Decimal( (1, (4, 3, 4, 9, 1, 3, 5, 3, 4), -25) )
        self.assertEqual(str(d), '-4.34913534E-17')
        d = nc.create_decimal( (1, (4, 3, 4, 9, 1, 3, 5, 3, 4), -25) )
        self.assertEqual(str(d), '-4.35E-17')

        # from Decimal
        prevdec = Decimal(500000123)
        d = Decimal(prevdec)
        self.assertEqual(str(d), '500000123')
        d = nc.create_decimal(prevdec)
        self.assertEqual(str(d), '5.00E+8')


class DecimalImplicitConstructionTest(unittest.TestCase):
    '''Unit tests for Implicit Construction cases of Decimal.'''

    def test_implicit_from_None(self):
        self.assertRaises(TypeError, eval, 'Decimal(5) + None', globals())

    def test_implicit_from_int(self):
        #normal
        self.assertEqual(str(Decimal(5) + 45), '50')
        #exceeding precision
        self.assertEqual(Decimal(5) + 123456789000, Decimal(123456789000))

    def test_implicit_from_string(self):
        self.assertRaises(TypeError, eval, 'Decimal(5) + "3"', globals())

    def test_implicit_from_float(self):
        self.assertRaises(TypeError, eval, 'Decimal(5) + 2.2', globals())

    def test_implicit_from_Decimal(self):
        self.assertEqual(Decimal(5) + Decimal(45), Decimal(50))


class DecimalArithmeticOperatorsTest(unittest.TestCase):
    '''Unit tests for all arithmetic operators, binary and unary.'''

    def test_addition(self):

        d1 = Decimal('-11.1')
        d2 = Decimal('22.2')

        #two Decimals
        self.assertEqual(d1+d2, Decimal('11.1'))
        self.assertEqual(d2+d1, Decimal('11.1'))

        #with other type, left
        c = d1 + 5
        self.assertEqual(c, Decimal('-6.1'))
        self.assertEqual(type(c), type(d1))

        #with other type, right
        c = 5 + d1
        self.assertEqual(c, Decimal('-6.1'))
        self.assertEqual(type(c), type(d1))

        #inline with decimal
        d1 += d2
        self.assertEqual(d1, Decimal('11.1'))

        #inline with other type
        d1 += 5
        self.assertEqual(d1, Decimal('16.1'))

    def test_subtraction(self):

        d1 = Decimal('-11.1')
        d2 = Decimal('22.2')

        #two Decimals
        self.assertEqual(d1-d2, Decimal('-33.3'))
        self.assertEqual(d2-d1, Decimal('33.3'))

        #with other type, left
        c = d1 - 5
        self.assertEqual(c, Decimal('-16.1'))
        self.assertEqual(type(c), type(d1))

        #with other type, right
        c = 5 - d1
        self.assertEqual(c, Decimal('16.1'))
        self.assertEqual(type(c), type(d1))

        #inline with decimal
        d1 -= d2
        self.assertEqual(d1, Decimal('-33.3'))

        #inline with other type
        d1 -= 5
        self.assertEqual(d1, Decimal('-38.3'))

    def test_multiplication(self):

        d1 = Decimal('-5')
        d2 = Decimal('3')

        #two Decimals
        self.assertEqual(d1*d2, Decimal('-15'))
        self.assertEqual(d2*d1, Decimal('-15'))

        #with other type, left
        c = d1 * 5
        self.assertEqual(c, Decimal('-25'))
        self.assertEqual(type(c), type(d1))

        #with other type, right
        c = 5 * d1
        self.assertEqual(c, Decimal('-25'))
        self.assertEqual(type(c), type(d1))

        #inline with decimal
        d1 *= d2
        self.assertEqual(d1, Decimal('-15'))

        #inline with other type
        d1 *= 5
        self.assertEqual(d1, Decimal('-75'))

    def test_division(self):

        d1 = Decimal('-5')
        d2 = Decimal('2')

        #two Decimals
        self.assertEqual(d1/d2, Decimal('-2.5'))
        self.assertEqual(d2/d1, Decimal('-0.4'))

        #with other type, left
        c = d1 / 4
        self.assertEqual(c, Decimal('-1.25'))
        self.assertEqual(type(c), type(d1))

        #with other type, right
        c = 4 / d1
        self.assertEqual(c, Decimal('-0.8'))
        self.assertEqual(type(c), type(d1))

        #inline with decimal
        d1 /= d2
        self.assertEqual(d1, Decimal('-2.5'))

        #inline with other type
        d1 /= 4
        self.assertEqual(d1, Decimal('-0.625'))

    def test_floor_division(self):

        d1 = Decimal('5')
        d2 = Decimal('2')

        #two Decimals
        self.assertEqual(d1//d2, Decimal('2'))
        self.assertEqual(d2//d1, Decimal('0'))

        #with other type, left
        c = d1 // 4
        self.assertEqual(c, Decimal('1'))
        self.assertEqual(type(c), type(d1))

        #with other type, right
        c = 7 // d1
        self.assertEqual(c, Decimal('1'))
        self.assertEqual(type(c), type(d1))

        #inline with decimal
        d1 //= d2
        self.assertEqual(d1, Decimal('2'))

        #inline with other type
        d1 //= 2
        self.assertEqual(d1, Decimal('1'))

    def test_powering(self):

        d1 = Decimal('5')
        d2 = Decimal('2')

        #two Decimals
        self.assertEqual(d1**d2, Decimal('25'))
        self.assertEqual(d2**d1, Decimal('32'))

        #with other type, left
        c = d1 ** 4
        self.assertEqual(c, Decimal('625'))
        self.assertEqual(type(c), type(d1))

        #with other type, right
        c = 7 ** d1
        self.assertEqual(c, Decimal('16807'))
        self.assertEqual(type(c), type(d1))

        #inline with decimal
        d1 **= d2
        self.assertEqual(d1, Decimal('25'))

        #inline with other type
        d1 **= 4
        self.assertEqual(d1, Decimal('390625'))

    def test_module(self):

        d1 = Decimal('5')
        d2 = Decimal('2')

        #two Decimals
        self.assertEqual(d1%d2, Decimal('1'))
        self.assertEqual(d2%d1, Decimal('2'))

        #with other type, left
        c = d1 % 4
        self.assertEqual(c, Decimal('1'))
        self.assertEqual(type(c), type(d1))

        #with other type, right
        c = 7 % d1
        self.assertEqual(c, Decimal('2'))
        self.assertEqual(type(c), type(d1))

        #inline with decimal
        d1 %= d2
        self.assertEqual(d1, Decimal('1'))

        #inline with other type
        d1 %= 4
        self.assertEqual(d1, Decimal('1'))

    def test_floor_div_module(self):

        d1 = Decimal('5')
        d2 = Decimal('2')

        #two Decimals
        (p, q) = divmod(d1, d2)
        self.assertEqual(p, Decimal('2'))
        self.assertEqual(q, Decimal('1'))
        self.assertEqual(type(p), type(d1))
        self.assertEqual(type(q), type(d1))

        #with other type, left
        (p, q) = divmod(d1, 4)
        self.assertEqual(p, Decimal('1'))
        self.assertEqual(q, Decimal('1'))
        self.assertEqual(type(p), type(d1))
        self.assertEqual(type(q), type(d1))

        #with other type, right
        (p, q) = divmod(7, d1)
        self.assertEqual(p, Decimal('1'))
        self.assertEqual(q, Decimal('2'))
        self.assertEqual(type(p), type(d1))
        self.assertEqual(type(q), type(d1))

    def test_unary_operators(self):
        self.assertEqual(+Decimal(45), Decimal(+45))           #  +
        self.assertEqual(-Decimal(45), Decimal(-45))           #  -
        self.assertEqual(abs(Decimal(45)), abs(Decimal(-45)))  # abs


# The following are two functions used to test threading in the next class

def thfunc1(cls):
    d1 = Decimal(1)
    d3 = Decimal(3)
    cls.assertEqual(d1/d3, Decimal('0.333333333'))
    cls.synchro.wait()
    cls.assertEqual(d1/d3, Decimal('0.333333333'))
    cls.finish1.set()
    return

def thfunc2(cls):
    d1 = Decimal(1)
    d3 = Decimal(3)
    cls.assertEqual(d1/d3, Decimal('0.333333333'))
    thiscontext = getcontext()
    thiscontext.prec = 18
    cls.assertEqual(d1/d3, Decimal('0.333333333333333333'))
    cls.synchro.set()
    cls.finish2.set()
    return


class DecimalUseOfContextTest(unittest.TestCase):
    '''Unit tests for Use of Context cases in Decimal.'''

    import threading
    # Take care executing this test from IDLE, there's an issue in threading
    # that hangs IDLE and I couldn't find it

    def test_threading(self):
        #Test the "threading isolation" of a Context.

        self.synchro = threading.Event()
        self.finish1 = threading.Event()
        self.finish2 = threading.Event()

        th1 = threading.Thread(target=thfunc1, args=(self,))
        th2 = threading.Thread(target=thfunc2, args=(self,))

        th1.start()
        th2.start()

        self.finish1.wait()
        self.finish1.wait()
        return


class DecimalUsabilityTest(unittest.TestCase):
    '''Unit tests for Usability cases of Decimal.'''

    def test_comparison_operators(self):

        da = Decimal('23.42')
        db = Decimal('23.42')
        dc = Decimal('45')

        #two Decimals
        self.failUnless(dc > da)
        self.failUnless(dc >= da)
        self.failUnless(da < dc)
        self.failUnless(da <= dc)
        self.failUnless(da == db)
        self.failUnless(da != dc)
        self.failUnless(da <= db)
        self.failUnless(da >= db)
        self.assertEqual(cmp(dc,da), 1)
        self.assertEqual(cmp(da,dc), -1)
        self.assertEqual(cmp(da,db), 0)

        #a Decimal and an int
        self.failUnless(dc > 23)
        self.failUnless(23 < dc)
        self.failUnless(dc == 45)
        self.assertEqual(cmp(dc,23), 1)
        self.assertEqual(cmp(23,dc), -1)
        self.assertEqual(cmp(dc,45), 0)

        #a Decimal and uncomparable
        self.assertNotEqual(da, 'ugly')
        self.assertNotEqual(da, 32.7)
        self.assertNotEqual(da, object())
        self.assertNotEqual(da, object)

        # sortable
        a = map(Decimal, xrange(100))
        b =  a[:]
        random.shuffle(a)
        a.sort()
        self.assertEqual(a, b)

    def test_copy_and_deepcopy_methods(self):
        d = Decimal('43.24')
        c = copy.copy(d)
        self.assertEqual(id(c), id(d))
        dc = copy.deepcopy(d)
        self.assertEqual(id(dc), id(d))

    def test_hash_method(self):
        #just that it's hashable
        hash(Decimal(23))
        #the same hash that to an int
        self.assertEqual(hash(Decimal(23)), hash(23))

    def test_min_and_max_methods(self):

        d1 = Decimal('15.32')
        d2 = Decimal('28.5')
        l1 = 15
        l2 = 28

        #between Decimals
        self.failUnless(min(d1,d2) is d1)
        self.failUnless(min(d2,d1) is d1)
        self.failUnless(max(d1,d2) is d2)
        self.failUnless(max(d2,d1) is d2)

        #between Decimal and long
        self.failUnless(min(d1,l2) is d1)
        self.failUnless(min(l2,d1) is d1)
        self.failUnless(max(l1,d2) is d2)
        self.failUnless(max(d2,l1) is d2)

    def test_as_nonzero(self):
        #as false
        self.failIf(Decimal(0))
        #as true
        self.failUnless(Decimal('0.372'))

    def test_tostring_methods(self):
        #Test str and repr methods.

        d = Decimal('15.32')
        self.assertEqual(str(d), '15.32')               # str
        self.assertEqual(repr(d), 'Decimal("15.32")')   # repr

    def test_tonum_methods(self):
        #Test float, int and long methods.

        d1 = Decimal('66')
        d2 = Decimal('15.32')

        #int
        self.assertEqual(int(d1), 66)
        self.assertEqual(int(d2), 15)

        #long
        self.assertEqual(long(d1), 66)
        self.assertEqual(long(d2), 15)

        #float
        self.assertEqual(float(d1), 66)
        self.assertEqual(float(d2), 15.32)

    def test_eval_round_trip(self):

        #with zero
        d = Decimal( (0, (0,), 0) )
        self.assertEqual(d, eval(repr(d)))

        #int
        d = Decimal( (1, (4, 5), 0) )
        self.assertEqual(d, eval(repr(d)))

        #float
        d = Decimal( (0, (4, 5, 3, 4), -2) )
        self.assertEqual(d, eval(repr(d)))

        #weird
        d = Decimal( (1, (4, 3, 4, 9, 1, 3, 5, 3, 4), -25) )
        self.assertEqual(d, eval(repr(d)))

    def test_as_tuple(self):

        #with zero
        d = Decimal(0)
        self.assertEqual(d.as_tuple(), (0, (0,), 0) )

        #int
        d = Decimal(-45)
        self.assertEqual(d.as_tuple(), (1, (4, 5), 0) )

        #complicated string
        d = Decimal("-4.34913534E-17")
        self.assertEqual(d.as_tuple(), (1, (4, 3, 4, 9, 1, 3, 5, 3, 4), -25) )

        #inf
        d = Decimal("Infinity")
        self.assertEqual(d.as_tuple(), (0, (0,), 'F') )

    def test_immutability_operations(self):
        # Do operations and check that it didn't change change internal objects.

        d1 = Decimal('-25e55')
        b1 = Decimal('-25e55')
        d2 = Decimal('33e-33')
        b2 = Decimal('33e-33')

        def checkSameDec(operation, useOther=False):
            if useOther:
                eval("d1." + operation + "(d2)")
                self.assertEqual(d1._sign, b1._sign)
                self.assertEqual(d1._int, b1._int)
                self.assertEqual(d1._exp, b1._exp)
                self.assertEqual(d2._sign, b2._sign)
                self.assertEqual(d2._int, b2._int)
                self.assertEqual(d2._exp, b2._exp)
            else:
                eval("d1." + operation + "()")
                self.assertEqual(d1._sign, b1._sign)
                self.assertEqual(d1._int, b1._int)
                self.assertEqual(d1._exp, b1._exp)
            return

        Decimal(d1)
        self.assertEqual(d1._sign, b1._sign)
        self.assertEqual(d1._int, b1._int)
        self.assertEqual(d1._exp, b1._exp)

        checkSameDec("__abs__")
        checkSameDec("__add__", True)
        checkSameDec("__div__", True)
        checkSameDec("__divmod__", True)
        checkSameDec("__cmp__", True)
        checkSameDec("__float__")
        checkSameDec("__floordiv__", True)
        checkSameDec("__hash__")
        checkSameDec("__int__")
        checkSameDec("__long__")
        checkSameDec("__mod__", True)
        checkSameDec("__mul__", True)
        checkSameDec("__neg__")
        checkSameDec("__nonzero__")
        checkSameDec("__pos__")
        checkSameDec("__pow__", True)
        checkSameDec("__radd__", True)
        checkSameDec("__rdiv__", True)
        checkSameDec("__rdivmod__", True)
        checkSameDec("__repr__")
        checkSameDec("__rfloordiv__", True)
        checkSameDec("__rmod__", True)
        checkSameDec("__rmul__", True)
        checkSameDec("__rpow__", True)
        checkSameDec("__rsub__", True)
        checkSameDec("__str__")
        checkSameDec("__sub__", True)
        checkSameDec("__truediv__", True)
        checkSameDec("adjusted")
        checkSameDec("as_tuple")
        checkSameDec("compare", True)
        checkSameDec("max", True)
        checkSameDec("min", True)
        checkSameDec("normalize")
        checkSameDec("quantize", True)
        checkSameDec("remainder_near", True)
        checkSameDec("same_quantum", True)
        checkSameDec("sqrt")
        checkSameDec("to_eng_string")
        checkSameDec("to_integral")

class DecimalPythonAPItests(unittest.TestCase):

    def test_pickle(self):
        d = Decimal('-3.141590000')
        p = pickle.dumps(d)
        e = pickle.loads(p)
        self.assertEqual(d, e)

    def test_int(self):
        data = '1.0 1.1 1.9 2.0 0.0 -1.0 -1.1 -1.9 -2.0'.split()
        for s in data:
            # should work the same as for floats
            self.assertEqual(int(Decimal(s)), int(float(s)))
            # should work the same as ROUND_DOWN
            d = Decimal(s)
            r = Context(prec=1, rounding=ROUND_DOWN).create_decimal(s)
            self.assertEqual(Decimal(int(d)), r)

class ContextAPItests(unittest.TestCase):

    def test_pickle(self):
        c = Context()
        e = pickle.loads(pickle.dumps(c))
        for k in vars(c):
            v1 = vars(c)[k]
            v2 = vars(e)[k]
            self.assertEqual(v1, v2)

    def test_equality_with_other_types(self):
        self.assert_(Decimal(10) in ['a', 1.0, Decimal(10), (1,2), {}])
        self.assert_(Decimal(10) not in ['a', 1.0, (1,2), {}])

    def test_copy(self):
        # All copies should be deep
        c = Context()
        d = c.copy()
        self.assertNotEqual(id(c), id(d))
        self.assertNotEqual(id(c.flags), id(d.flags))
        self.assertNotEqual(id(c.traps), id(d.traps))

def test_main(arith=False, verbose=None):
    """ Execute the tests.

    Runs arithmetic tests if arith is True or if the "decimal" resource
    is enables in regrtest.py
    """
    test_classes = [
        DecimalExplicitConstructionTest,
        DecimalImplicitConstructionTest,
        DecimalArithmeticOperatorsTest,
        DecimalUseOfContextTest,
        DecimalUsabilityTest,
        DecimalPythonAPItests,
        ContextAPItests,
    ]

    if arith or is_resource_enabled('decimal'):
        test_classes.extend([DecimalTest])

    run_unittest(*test_classes)
    import decimal as DecimalModule
    run_doctest(DecimalModule, verbose)
    return


if __name__ == '__main__':
    # Calling with no arguments runs all tests.
    # Calling with "Skip" will skipover the arithmetic tests.
    if len(sys.argv) == 1:
        test_main(arith=True, verbose=True)
    elif len(sys.argv) == 2:
        arith = sys.argv[1].lower() != 'skip'
        test_main(arith=arith, verbose=True)
    else:
        raise ValueError("test called with wrong arguments, use test_Decimal [Skip]")



"""distutils.command.x

Implements the Distutils 'x' command.
"""

# created 2000/mm/dd, John Doe

__revision__ = "$Id: checkdep.py,v 1.1 2004/03/22 22:22:05 anthonybaxter Exp $"

from distutils.core import Command

class DependencyFailure(Exception): pass

class VersionTooOld(DependencyFailure): pass

class VersionNotKnown(DependencyFailure): pass

class checkdep (Command):

    # Brief (40-50 characters) description of the command
    description = "check package dependencies"

    # List of option tuples: long name, short name (None if no short
    # name), and help string.
    # Later on, we might have auto-fetch and the like here. Feel free.
    user_options = []

    def initialize_options (self):
        self.debug = None

    # initialize_options()


    def finalize_options (self):
        pass
    # finalize_options()


    def run (self):
        from distutils.version import LooseVersion
        failed = []
        for pkg, ver in self.distribution.metadata.requires:
            if pkg == 'python':
                if ver is not None:
                    # Special case the 'python' package
                    import sys
                    thisver = LooseVersion('%d.%d.%d'%sys.version_info[:3])
                    if thisver < ver:
                        failed.append(((pkg,ver), VersionTooOld(thisver)))
                continue
            # Kinda hacky - we should do more here
            try:
                mod = __import__(pkg)
            except Exception, e:
                failed.append(((pkg,ver), e))
                continue
            if ver is not None:
                if hasattr(mod, '__version__'):
                    thisver = LooseVersion(mod.__version__)
                    if thisver < ver:
                        failed.append(((pkg,ver), VersionTooOld(thisver)))
                else:
                    failed.append(((pkg,ver), VersionNotKnown()))

        if failed:
            raise DependencyFailure, failed

    # run()

# class x



"""An object-oriented interface to .netrc files."""

# Module and documentation by Eric S. Raymond, 21 Dec 1998

import os, shlex

__all__ = ["netrc", "NetrcParseError"]


class NetrcParseError(Exception):
    """Exception raised on syntax errors in the .netrc file."""
    def __init__(self, msg, filename=None, lineno=None):
        self.filename = filename
        self.lineno = lineno
        self.msg = msg
        Exception.__init__(self, msg)

    def __str__(self):
        return "%s (%s, line %s)" % (self.msg, self.filename, self.lineno)


class netrc:
    def __init__(self, file=None):
        if file is None:
            try:
                file = os.path.join(os.environ['HOME'], ".netrc")
            except KeyError:
                raise IOError("Could not find .netrc: $HOME is not set")
        fp = open(file)
        self.hosts = {}
        self.macros = {}
        lexer = shlex.shlex(fp)
        lexer.wordchars += r"""!"#$%&'()*+,-./:;<=>?@[\]^_`{|}~"""
        while 1:
            # Look for a machine, default, or macdef top-level keyword
            toplevel = tt = lexer.get_token()
            if not tt:
                break
            elif tt == 'machine':
                entryname = lexer.get_token()
            elif tt == 'default':
                entryname = 'default'
            elif tt == 'macdef':                # Just skip to end of macdefs
                entryname = lexer.get_token()
                self.macros[entryname] = []
                lexer.whitespace = ' \t'
                while 1:
                    line = lexer.instream.readline()
                    if not line or line == '\012':
                        lexer.whitespace = ' \t\r\n'
                        break
                    self.macros[entryname].append(line)
                continue
            else:
                raise NetrcParseError(
                    "bad toplevel token %r" % tt, file, lexer.lineno)

            # We're looking at start of an entry for a named machine or default.
            login = ''
            account = password = None
            self.hosts[entryname] = {}
            while 1:
                tt = lexer.get_token()
                if (tt=='' or tt == 'machine' or
                    tt == 'default' or tt =='macdef'):
                    if password:
                        self.hosts[entryname] = (login, account, password)
                        lexer.push_token(tt)
                        break
                    else:
                        raise NetrcParseError(
                            "malformed %s entry %s terminated by %s"
                            % (toplevel, entryname, repr(tt)),
                            file, lexer.lineno)
                elif tt == 'login' or tt == 'user':
                    login = lexer.get_token()
                elif tt == 'account':
                    account = lexer.get_token()
                elif tt == 'password':
                    password = lexer.get_token()
                else:
                    raise NetrcParseError("bad follower token %r" % tt,
                                          file, lexer.lineno)

    def authenticators(self, host):
        """Return a (user, account, password) tuple for given host."""
        if host in self.hosts:
            return self.hosts[host]
        elif 'default' in self.hosts:
            return self.hosts['default']
        else:
            return None

    def __repr__(self):
        """Dump the class data in the format of a .netrc file."""
        rep = ""
        for host in self.hosts.keys():
            attrs = self.hosts[host]
            rep = rep + "machine "+ host + "\n\tlogin " + repr(attrs[0]) + "\n"
            if attrs[1]:
                rep = rep + "account " + repr(attrs[1])
            rep = rep + "\tpassword " + repr(attrs[2]) + "\n"
        for macro in self.macros.keys():
            rep = rep + "macdef " + macro + "\n"
            for line in self.macros[macro]:
                rep = rep + line
            rep = rep + "\n"
        return rep

if __name__ == '__main__':
    print netrc()



import struct, fcntl

def writelock(f):
    _lock(f, fcntl.F_WRLCK)

def readlock(f):
    _lock(f, fcntl.F_RDLCK)

def unlock(f):
    _lock(f, fcntl.F_UNLCK)

def _lock(f, op):
    dummy = fcntl.fcntl(f.fileno(), fcntl.F_SETLKW,
                        struct.pack('2h8l', op,
                                    0, 0, 0, 0, 0, 0, 0, 0, 0))



# Module 'packmail' -- create a self-unpacking shell archive.

# This module works on UNIX and on the Mac; the archives can unpack
# themselves only on UNIX.

import os
from stat import ST_MTIME

# Print help
def help():
    print 'All fns have a file open for writing as first parameter'
    print 'pack(f, fullname, name): pack fullname as name'
    print 'packsome(f, directory, namelist): selected files from directory'
    print 'packall(f, directory): pack all files from directory'
    print 'packnotolder(f, directory, name): pack all files from directory'
    print '                        that are not older than a file there'
    print 'packtree(f, directory): pack entire directory tree'

# Pack one file
def pack(outfp, file, name):
    fp = open(file, 'r')
    outfp.write('echo ' + name + '\n')
    outfp.write('sed "s/^X//" >"' + name + '" <<"!"\n')
    while 1:
        line = fp.readline()
        if not line: break
        if line[-1:] != '\n':
            line = line + '\n'
        outfp.write('X' + line)
    outfp.write('!\n')
    fp.close()

# Pack some files from a directory
def packsome(outfp, dirname, names):
    for name in names:
        print name
        file = os.path.join(dirname, name)
        pack(outfp, file, name)

# Pack all files from a directory
def packall(outfp, dirname):
    names = os.listdir(dirname)
    try:
        names.remove('.')
    except:
        pass
    try:
        names.remove('..')
    except:
        pass
    names.sort()
    packsome(outfp, dirname, names)

# Pack all files from a directory that are not older than a give one
def packnotolder(outfp, dirname, oldest):
    names = os.listdir(dirname)
    try:
        names.remove('.')
    except:
        pass
    try:
        names.remove('..')
    except:
        pass
    oldest = os.path.join(dirname, oldest)
    st = os.stat(oldest)
    mtime = st[ST_MTIME]
    todo = []
    for name in names:
        print name, '...',
        st = os.stat(os.path.join(dirname, name))
        if st[ST_MTIME] >= mtime:
            print 'Yes.'
            todo.append(name)
        else:
            print 'No.'
    todo.sort()
    packsome(outfp, dirname, todo)

# Pack a whole tree (no exceptions)
def packtree(outfp, dirname):
    print 'packtree', dirname
    outfp.write('mkdir ' + unixfix(dirname) + '\n')
    names = os.listdir(dirname)
    try:
        names.remove('.')
    except:
        pass
    try:
        names.remove('..')
    except:
        pass
    subdirs = []
    for name in names:
        fullname = os.path.join(dirname, name)
        if os.path.isdir(fullname):
            subdirs.append(fullname)
        else:
            print 'pack', fullname
            pack(outfp, fullname, unixfix(fullname))
    for subdirname in subdirs:
        packtree(outfp, subdirname)

def unixfix(name):
    comps = name.split(os.sep)
    res = ''
    for comp in comps:
        if comp:
            if res: res = res + '/'
            res = res + comp
    return res



#!/usr/bin/env python
#
# test_multibytecodec.py
#   Unit test for multibytecodec itself
#
# $CJKCodecs: test_multibytecodec.py,v 1.8 2004/06/19 06:09:55 perky Exp $

from test import test_support
from test import test_multibytecodec_support
import unittest, StringIO, codecs

class Test_StreamWriter(unittest.TestCase):
    if len(u'\U00012345') == 2: # UCS2
        def test_gb18030(self):
            s= StringIO.StringIO()
            c = codecs.lookup('gb18030')[3](s)
            c.write(u'123')
            self.assertEqual(s.getvalue(), '123')
            c.write(u'\U00012345')
            self.assertEqual(s.getvalue(), '123\x907\x959')
            c.write(u'\U00012345'[0])
            self.assertEqual(s.getvalue(), '123\x907\x959')
            c.write(u'\U00012345'[1] + u'\U00012345' + u'\uac00\u00ac')
            self.assertEqual(s.getvalue(),
                    '123\x907\x959\x907\x959\x907\x959\x827\xcf5\x810\x851')
            c.write(u'\U00012345'[0])
            self.assertEqual(s.getvalue(),
                    '123\x907\x959\x907\x959\x907\x959\x827\xcf5\x810\x851')
            self.assertRaises(UnicodeError, c.reset)
            self.assertEqual(s.getvalue(),
                    '123\x907\x959\x907\x959\x907\x959\x827\xcf5\x810\x851')

        # standard utf-8 codecs has broken StreamReader
        if test_multibytecodec_support.__cjkcodecs__:
            def test_utf_8(self):
                s= StringIO.StringIO()
                c = codecs.lookup('utf-8')[3](s)
                c.write(u'123')
                self.assertEqual(s.getvalue(), '123')
                c.write(u'\U00012345')
                self.assertEqual(s.getvalue(), '123\xf0\x92\x8d\x85')
                c.write(u'\U00012345'[0])
                self.assertEqual(s.getvalue(), '123\xf0\x92\x8d\x85')
                c.write(u'\U00012345'[1] + u'\U00012345' + u'\uac00\u00ac')
                self.assertEqual(s.getvalue(),
                    '123\xf0\x92\x8d\x85\xf0\x92\x8d\x85\xf0\x92\x8d\x85'
                    '\xea\xb0\x80\xc2\xac')
                c.write(u'\U00012345'[0])
                self.assertEqual(s.getvalue(),
                    '123\xf0\x92\x8d\x85\xf0\x92\x8d\x85\xf0\x92\x8d\x85'
                    '\xea\xb0\x80\xc2\xac')
                c.reset()
                self.assertEqual(s.getvalue(),
                    '123\xf0\x92\x8d\x85\xf0\x92\x8d\x85\xf0\x92\x8d\x85'
                    '\xea\xb0\x80\xc2\xac\xed\xa0\x88')
                c.write(u'\U00012345'[1])
                self.assertEqual(s.getvalue(),
                    '123\xf0\x92\x8d\x85\xf0\x92\x8d\x85\xf0\x92\x8d\x85'
                    '\xea\xb0\x80\xc2\xac\xed\xa0\x88\xed\xbd\x85')

    else: # UCS4
        pass

    def test_nullcoding(self):
        self.assertEqual(''.decode('gb18030'), u'')
        self.assertEqual(unicode('', 'gb18030'), u'')
        self.assertEqual(u''.encode('gb18030'), '')

    def test_str_decode(self):
        self.assertEqual('abcd'.encode('gb18030'), 'abcd')

    def test_streamwriter_strwrite(self):
        s = StringIO.StringIO()
        wr = codecs.getwriter('gb18030')(s)
        wr.write('abcd')
        self.assertEqual(s.getvalue(), 'abcd')

def test_main():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test_StreamWriter))
    test_support.run_suite(suite)

if __name__ == "__main__":
    test_main()



"""A (less & less) simple Python editor"""

import W
import Wtraceback
from Wkeys import *

import MacOS
import EasyDialogs
from Carbon import Win
from Carbon import Res
from Carbon import Evt
from Carbon import Qd
from Carbon import File
import os
import imp
import sys
import string
import marshal
import re

smAllScripts = -3

if hasattr(Win, "FrontNonFloatingWindow"):
    MyFrontWindow = Win.FrontNonFloatingWindow
else:
    MyFrontWindow = Win.FrontWindow


_scriptuntitledcounter = 1
_wordchars = string.ascii_letters + string.digits + "_"


runButtonLabels = ["Run all", "Stop!"]
runSelButtonLabels = ["Run selection", "Pause!", "Resume"]


class Editor(W.Window):

    def __init__(self, path = "", title = ""):
        defaultfontsettings, defaulttabsettings, defaultwindowsize = geteditorprefs()
        global _scriptuntitledcounter
        if not path:
            if title:
                self.title = title
            else:
                self.title = "Untitled Script %r" % (_scriptuntitledcounter,)
                _scriptuntitledcounter = _scriptuntitledcounter + 1
            text = ""
            self._creator = W._signature
            self._eoln = os.linesep
        elif os.path.exists(path):
            path = resolvealiases(path)
            dir, name = os.path.split(path)
            self.title = name
            f = open(path, "rb")
            text = f.read()
            f.close()
            self._creator, filetype = MacOS.GetCreatorAndType(path)
            self.addrecentfile(path)
            if '\n' in text:
                if string.find(text, '\r\n') >= 0:
                    self._eoln = '\r\n'
                else:
                    self._eoln = '\n'
                text = string.replace(text, self._eoln, '\r')
            else:
                self._eoln = '\r'
        else:
            raise IOError, "file '%s' does not exist" % path
        self.path = path

        self.settings = {}
        if self.path:
            self.readwindowsettings()
        if self.settings.has_key("windowbounds"):
            bounds = self.settings["windowbounds"]
        else:
            bounds = defaultwindowsize
        if self.settings.has_key("fontsettings"):
            self.fontsettings = self.settings["fontsettings"]
        else:
            self.fontsettings = defaultfontsettings
        if self.settings.has_key("tabsize"):
            try:
                self.tabsettings = (tabsize, tabmode) = self.settings["tabsize"]
            except:
                self.tabsettings = defaulttabsettings
        else:
            self.tabsettings = defaulttabsettings

        W.Window.__init__(self, bounds, self.title, minsize = (330, 120), tabbable = 0)
        self.setupwidgets(text)

        if self.settings.has_key("selection"):
            selstart, selend = self.settings["selection"]
            self.setselection(selstart, selend)
        self.open()
        self.setinfotext()
        self.globals = {}
        self._buf = ""  # for write method
        self.debugging = 0
        self.profiling = 0
        self.run_as_main = self.settings.get("run_as_main", 0)
        self.run_with_interpreter = self.settings.get("run_with_interpreter", 0)
        self.run_with_cl_interpreter = self.settings.get("run_with_cl_interpreter", 0)

    def readwindowsettings(self):
        try:
            resref = Res.FSpOpenResFile(self.path, 1)
        except Res.Error:
            return
        try:
            Res.UseResFile(resref)
            data = Res.Get1Resource('PyWS', 128)
            self.settings = marshal.loads(data.data)
        except:
            pass
        Res.CloseResFile(resref)

    def writewindowsettings(self):
        try:
            resref = Res.FSpOpenResFile(self.path, 3)
        except Res.Error:
            Res.FSpCreateResFile(self.path, self._creator, 'TEXT', smAllScripts)
            resref = Res.FSpOpenResFile(self.path, 3)
        try:
            data = Res.Resource(marshal.dumps(self.settings))
            Res.UseResFile(resref)
            try:
                temp = Res.Get1Resource('PyWS', 128)
                temp.RemoveResource()
            except Res.Error:
                pass
            data.AddResource('PyWS', 128, "window settings")
        finally:
            Res.UpdateResFile(resref)
            Res.CloseResFile(resref)

    def getsettings(self):
        self.settings = {}
        self.settings["windowbounds"] = self.getbounds()
        self.settings["selection"] = self.getselection()
        self.settings["fontsettings"] = self.editgroup.editor.getfontsettings()
        self.settings["tabsize"] = self.editgroup.editor.gettabsettings()
        self.settings["run_as_main"] = self.run_as_main
        self.settings["run_with_interpreter"] = self.run_with_interpreter
        self.settings["run_with_cl_interpreter"] = self.run_with_cl_interpreter

    def get(self):
        return self.editgroup.editor.get()

    def getselection(self):
        return self.editgroup.editor.ted.WEGetSelection()

    def setselection(self, selstart, selend):
        self.editgroup.editor.setselection(selstart, selend)

    def getselectedtext(self):
        return self.editgroup.editor.getselectedtext()

    def getfilename(self):
        if self.path:
            return self.path
        return '<%s>' % self.title

    def setupwidgets(self, text):
        topbarheight = 24
        popfieldwidth = 80
        self.lastlineno = None

        # make an editor
        self.editgroup = W.Group((0, topbarheight + 1, 0, 0))
        editor = W.PyEditor((0, 0, -15,-15), text,
                        fontsettings = self.fontsettings,
                        tabsettings = self.tabsettings,
                        file = self.getfilename())

        # make the widgets
        self.popfield = ClassFinder((popfieldwidth - 17, -15, 16, 16), [], self.popselectline)
        self.linefield = W.EditText((-1, -15, popfieldwidth - 15, 16), inset = (6, 1))
        self.editgroup._barx = W.Scrollbar((popfieldwidth - 2, -15, -14, 16), editor.hscroll, max = 32767)
        self.editgroup._bary = W.Scrollbar((-15, 14, 16, -14), editor.vscroll, max = 32767)
        self.editgroup.editor = editor  # add editor *after* scrollbars

        self.editgroup.optionsmenu = W.PopupMenu((-15, -1, 16, 16), [])
        self.editgroup.optionsmenu.bind('<click>', self.makeoptionsmenu)

        self.bevelbox = W.BevelBox((0, 0, 0, topbarheight))
        self.hline = W.HorizontalLine((0, topbarheight, 0, 0))
        self.infotext = W.TextBox((175, 6, -4, 14), backgroundcolor = (0xe000, 0xe000, 0xe000))
        self.runbutton = W.BevelButton((6, 4, 80, 16), runButtonLabels[0], self.run)
        self.runselbutton = W.BevelButton((90, 4, 80, 16), runSelButtonLabels[0], self.runselection)

        # bind some keys
        editor.bind("cmdr", self.runbutton.push)
        editor.bind("enter", self.runselbutton.push)
        editor.bind("cmdj", self.domenu_gotoline)
        editor.bind("cmdd", self.domenu_toggledebugger)
        editor.bind("<idle>", self.updateselection)

        editor.bind("cmde", searchengine.setfindstring)
        editor.bind("cmdf", searchengine.show)
        editor.bind("cmdg", searchengine.findnext)
        editor.bind("cmdshiftr", searchengine.replace)
        editor.bind("cmdt", searchengine.replacefind)

        self.linefield.bind("return", self.dolinefield)
        self.linefield.bind("enter", self.dolinefield)
        self.linefield.bind("tab", self.dolinefield)

        # intercept clicks
        editor.bind("<click>", self.clickeditor)
        self.linefield.bind("<click>", self.clicklinefield)

    def makeoptionsmenu(self):
        menuitems = [('Font settings\xc9', self.domenu_fontsettings),
                        ("Save options\xc9", self.domenu_options),
                        '-',
                        ('\0' + chr(self.run_as_main) + 'Run as __main__', self.domenu_toggle_run_as_main),
                        #('\0' + chr(self.run_with_interpreter) + 'Run with Interpreter', self.domenu_dtoggle_run_with_interpreter),
                        ('\0' + chr(self.run_with_cl_interpreter) + 'Run with commandline Python', self.domenu_toggle_run_with_cl_interpreter),
                        '-',
                        ('Modularize', self.domenu_modularize),
                        ('Browse namespace\xc9', self.domenu_browsenamespace),
                        '-']
        if self.profiling:
            menuitems = menuitems + [('Disable profiler', self.domenu_toggleprofiler)]
        else:
            menuitems = menuitems + [('Enable profiler', self.domenu_toggleprofiler)]
        if self.editgroup.editor._debugger:
            menuitems = menuitems + [('Disable debugger', self.domenu_toggledebugger),
                    ('Clear breakpoints', self.domenu_clearbreakpoints),
                    ('Edit breakpoints\xc9', self.domenu_editbreakpoints)]
        else:
            menuitems = menuitems + [('Enable debugger', self.domenu_toggledebugger)]
        self.editgroup.optionsmenu.set(menuitems)

    def domenu_toggle_run_as_main(self):
        self.run_as_main = not self.run_as_main
        self.run_with_interpreter = 0
        self.run_with_cl_interpreter = 0
        self.editgroup.editor.selectionchanged()

    def XXdomenu_toggle_run_with_interpreter(self):
        self.run_with_interpreter = not self.run_with_interpreter
        self.run_as_main = 0
        self.run_with_cl_interpreter = 0
        self.editgroup.editor.selectionchanged()

    def domenu_toggle_run_with_cl_interpreter(self):
        self.run_with_cl_interpreter = not self.run_with_cl_interpreter
        self.run_as_main = 0
        self.run_with_interpreter = 0
        self.editgroup.editor.selectionchanged()

    def showbreakpoints(self, onoff):
        self.editgroup.editor.showbreakpoints(onoff)
        self.debugging = onoff

    def domenu_clearbreakpoints(self, *args):
        self.editgroup.editor.clearbreakpoints()

    def domenu_editbreakpoints(self, *args):
        self.editgroup.editor.editbreakpoints()

    def domenu_toggledebugger(self, *args):
        if not self.debugging:
            W.SetCursor('watch')
        self.debugging = not self.debugging
        self.editgroup.editor.togglebreakpoints()

    def domenu_toggleprofiler(self, *args):
        self.profiling = not self.profiling

    def domenu_browsenamespace(self, *args):
        import PyBrowser, W
        W.SetCursor('watch')
        globals, file, modname = self.getenvironment()
        if not modname:
            modname = self.title
        PyBrowser.Browser(globals, "Object browser: " + modname)

    def domenu_modularize(self, *args):
        modname = _filename_as_modname(self.title)
        if not modname:
            raise W.AlertError, "Can't modularize \"%s\"" % self.title
        run_as_main = self.run_as_main
        self.run_as_main = 0
        self.run()
        self.run_as_main = run_as_main
        if self.path:
            file = self.path
        else:
            file = self.title

        if self.globals and not sys.modules.has_key(modname):
            module = imp.new_module(modname)
            for attr in self.globals.keys():
                setattr(module,attr,self.globals[attr])
            sys.modules[modname] = module
            self.globals = {}

    def domenu_fontsettings(self, *args):
        import FontSettings
        fontsettings = self.editgroup.editor.getfontsettings()
        tabsettings = self.editgroup.editor.gettabsettings()
        settings = FontSettings.FontDialog(fontsettings, tabsettings)
        if settings:
            fontsettings, tabsettings = settings
            self.editgroup.editor.setfontsettings(fontsettings)
            self.editgroup.editor.settabsettings(tabsettings)

    def domenu_options(self, *args):
        rv = SaveOptions(self._creator, self._eoln)
        if rv:
            self.editgroup.editor.selectionchanged() # ouch...
            self._creator, self._eoln = rv

    def clicklinefield(self):
        if self._currentwidget <> self.linefield:
            self.linefield.select(1)
            self.linefield.selectall()
            return 1

    def clickeditor(self):
        if self._currentwidget <> self.editgroup.editor:
            self.dolinefield()
            return 1

    def updateselection(self, force = 0):
        sel = min(self.editgroup.editor.getselection())
        lineno = self.editgroup.editor.offsettoline(sel)
        if lineno <> self.lastlineno or force:
            self.lastlineno = lineno
            self.linefield.set(str(lineno + 1))
            self.linefield.selview()

    def dolinefield(self):
        try:
            lineno = string.atoi(self.linefield.get()) - 1
            if lineno <> self.lastlineno:
                self.editgroup.editor.selectline(lineno)
                self.updateselection(1)
        except:
            self.updateselection(1)
        self.editgroup.editor.select(1)

    def setinfotext(self):
        if not hasattr(self, 'infotext'):
            return
        if self.path:
            self.infotext.set(self.path)
        else:
            self.infotext.set("")

    def close(self):
        if self.editgroup.editor.changed:
            Qd.InitCursor()
            save = EasyDialogs.AskYesNoCancel('Save window "%s" before closing?' % self.title,
                            default=1, no="Don\xd5t save")
            if save > 0:
                if self.domenu_save():
                    return 1
            elif save < 0:
                return 1
        self.globals = None
        W.Window.close(self)

    def domenu_close(self, *args):
        return self.close()

    def domenu_save(self, *args):
        if not self.path:
            # Will call us recursively
            return self.domenu_save_as()
        data = self.editgroup.editor.get()
        if self._eoln != '\r':
            data = string.replace(data, '\r', self._eoln)
        fp = open(self.path, 'wb')  # open file in binary mode, data has '\r' line-endings
        fp.write(data)
        fp.close()
        MacOS.SetCreatorAndType(self.path, self._creator, 'TEXT')
        self.getsettings()
        self.writewindowsettings()
        self.editgroup.editor.changed = 0
        self.editgroup.editor.selchanged = 0
        import linecache
        if linecache.cache.has_key(self.path):
            del linecache.cache[self.path]
        import macostools
        macostools.touched(self.path)
        self.addrecentfile(self.path)

    def can_save(self, menuitem):
        return self.editgroup.editor.changed or self.editgroup.editor.selchanged

    def domenu_save_as(self, *args):
        path = EasyDialogs.AskFileForSave(message='Save as:', savedFileName=self.title)
        if not path:
            return 1
        self.showbreakpoints(0)
        self.path = path
        self.setinfotext()
        self.title = os.path.split(self.path)[-1]
        self.wid.SetWTitle(self.title)
        self.domenu_save()
        self.editgroup.editor.setfile(self.getfilename())
        app = W.getapplication()
        app.makeopenwindowsmenu()
        if hasattr(app, 'makescriptsmenu'):
            app = W.getapplication()
            fsr, changed = app.scriptsfolder.FSResolveAlias(None)
            path = fsr.as_pathname()
            if path == self.path[:len(path)]:
                W.getapplication().makescriptsmenu()

    def domenu_save_as_applet(self, *args):
        import buildtools

        buildtools.DEBUG = 0    # ouch.

        if self.title[-3:] == ".py":
            destname = self.title[:-3]
        else:
            destname = self.title + ".applet"
        destname = EasyDialogs.AskFileForSave(message='Save as Applet:',
                savedFileName=destname)
        if not destname:
            return 1
        W.SetCursor("watch")
        if self.path:
            filename = self.path
            if filename[-3:] == ".py":
                rsrcname = filename[:-3] + '.rsrc'
            else:
                rsrcname = filename + '.rsrc'
        else:
            filename = self.title
            rsrcname = ""

        pytext = self.editgroup.editor.get()
        pytext = string.split(pytext, '\r')
        pytext = string.join(pytext, '\n') + '\n'
        try:
            code = compile(pytext, filename, "exec")
        except (SyntaxError, EOFError):
            raise buildtools.BuildError, "Syntax error in script %r" % (filename,)

        import tempfile
        tmpdir = tempfile.mkdtemp()

        if filename[-3:] != ".py":
            filename = filename + ".py"
        filename = os.path.join(tmpdir, os.path.split(filename)[1])
        fp = open(filename, "w")
        fp.write(pytext)
        fp.close()

        # Try removing the output file
        try:
            os.remove(destname)
        except os.error:
            pass
        template = buildtools.findtemplate()
        buildtools.process(template, filename, destname, 1, rsrcname=rsrcname, progress=None)
        try:
            os.remove(filename)
            os.rmdir(tmpdir)
        except os.error:
            pass

    def domenu_gotoline(self, *args):
        self.linefield.selectall()
        self.linefield.select(1)
        self.linefield.selectall()

    def domenu_selectline(self, *args):
        self.editgroup.editor.expandselection()

    def domenu_find(self, *args):
        searchengine.show()

    def domenu_entersearchstring(self, *args):
        searchengine.setfindstring()

    def domenu_replace(self, *args):
        searchengine.replace()

    def domenu_findnext(self, *args):
        searchengine.findnext()

    def domenu_replacefind(self, *args):
        searchengine.replacefind()

    def domenu_run(self, *args):
        self.runbutton.push()

    def domenu_runselection(self, *args):
        self.runselbutton.push()

    def run(self):
        self._run()

    def _run(self):
        if self.run_with_interpreter:
            if self.editgroup.editor.changed:
                Qd.InitCursor()
                save = EasyDialogs.AskYesNoCancel('Save "%s" before running?' % self.title, 1)
                if save > 0:
                    if self.domenu_save():
                        return
                elif save < 0:
                    return
            if not self.path:
                raise W.AlertError, "Can't run unsaved file"
            self._run_with_interpreter()
        elif self.run_with_cl_interpreter:
            if self.editgroup.editor.changed:
                Qd.InitCursor()
                save = EasyDialogs.AskYesNoCancel('Save "%s" before running?' % self.title, 1)
                if save > 0:
                    if self.domenu_save():
                        return
                elif save < 0:
                    return
            if not self.path:
                raise W.AlertError, "Can't run unsaved file"
            self._run_with_cl_interpreter()
        else:
            pytext = self.editgroup.editor.get()
            globals, file, modname = self.getenvironment()
            self.execstring(pytext, globals, globals, file, modname)

    def _run_with_interpreter(self):
        interp_path = os.path.join(sys.exec_prefix, "PythonInterpreter")
        if not os.path.exists(interp_path):
            raise W.AlertError, "Can't find interpreter"
        import findertools
        XXX

    def _run_with_cl_interpreter(self):
        import Terminal
        interp_path = os.path.join(sys.exec_prefix,
                "Resources", "Python.app", "Contents", "MacOS", "Python")
        if not os.path.exists(interp_path):
            interp_path = os.path.join(sys.exec_prefix, "bin", "python")
        file_path = self.path
        if not os.path.exists(interp_path):
            # This "can happen" if we are running IDE under MacPython-OS9.
            raise W.AlertError, "Can't find command-line Python"
        cmd = '"%s" "%s" ; exit' % (interp_path, file_path)
        t = Terminal.Terminal()
        t.do_script(cmd)

    def runselection(self):
        self._runselection()

    def _runselection(self):
        if self.run_with_interpreter or self.run_with_cl_interpreter:
            raise W.AlertError, "Can't run selection with Interpreter"
        globals, file, modname = self.getenvironment()
        locals = globals
        # select whole lines
        self.editgroup.editor.expandselection()

        # get lineno of first selected line
        selstart, selend = self.editgroup.editor.getselection()
        selstart, selend = min(selstart, selend), max(selstart, selend)
        selfirstline = self.editgroup.editor.offsettoline(selstart)
        alltext = self.editgroup.editor.get()
        pytext = alltext[selstart:selend]
        lines = string.split(pytext, '\r')
        indent = getminindent(lines)
        if indent == 1:
            classname = ''
            alllines = string.split(alltext, '\r')
            for i in range(selfirstline - 1, -1, -1):
                line = alllines[i]
                if line[:6] == 'class ':
                    classname = string.split(string.strip(line[6:]))[0]
                    classend = identifieRE_match(classname)
                    if classend < 1:
                        raise W.AlertError, "Can't find a class."
                    classname = classname[:classend]
                    break
                elif line and line[0] not in '\t#':
                    raise W.AlertError, "Can't find a class."
            else:
                raise W.AlertError, "Can't find a class."
            if globals.has_key(classname):
                klass = globals[classname]
            else:
                raise W.AlertError, "Can't find class \"%s\"." % classname
            # add class def
            pytext = ("class %s:\n" % classname) + pytext
            selfirstline = selfirstline - 1
        elif indent > 0:
            raise W.AlertError, "Can't run indented code."

        # add "newlines" to fool compile/exec:
        # now a traceback will give the right line number
        pytext = selfirstline * '\r' + pytext
        self.execstring(pytext, globals, locals, file, modname)
        if indent == 1 and globals[classname] is not klass:
            # update the class in place
            klass.__dict__.update(globals[classname].__dict__)
            globals[classname] = klass

    def execstring(self, pytext, globals, locals, file, modname):
        tracebackwindow.hide()
        # update windows
        W.getapplication().refreshwindows()
        if self.run_as_main:
            modname = "__main__"
        if self.path:
            dir = os.path.dirname(self.path)
            savedir = os.getcwd()
            os.chdir(dir)
            sys.path.insert(0, dir)
        self._scriptDone = False
        if sys.platform == "darwin":
            # On MacOSX, MacPython doesn't poll for command-period
            # (cancel), so to enable the user to cancel a running
            # script, we have to spawn a thread which does the
            # polling. It will send a SIGINT to the main thread
            # (in which the script is running) when the user types
            # command-period.
            from threading import Thread
            t = Thread(target=self._userCancelledMonitor,
                            name="UserCancelledMonitor")
            t.start()
        try:
            execstring(pytext, globals, locals, file, self.debugging,
                            modname, self.profiling)
        finally:
            self._scriptDone = True
            if self.path:
                os.chdir(savedir)
                del sys.path[0]

    def _userCancelledMonitor(self):
        import time
        from signal import SIGINT
        while not self._scriptDone:
            if Evt.CheckEventQueueForUserCancel():
                # Send a SIGINT signal to ourselves.
                # This gets delivered to the main thread,
                # cancelling the running script.
                os.kill(os.getpid(), SIGINT)
                break
            time.sleep(0.25)

    def getenvironment(self):
        if self.path:
            file = self.path
            dir = os.path.dirname(file)
            # check if we're part of a package
            modname = ""
            while os.path.exists(os.path.join(dir, "__init__.py")):
                dir, dirname = os.path.split(dir)
                modname = dirname + '.' + modname
            subname = _filename_as_modname(self.title)
            if subname is None:
                return self.globals, file, None
            if modname:
                if subname == "__init__":
                    # strip trailing period
                    modname = modname[:-1]
                else:
                    modname = modname + subname
            else:
                modname = subname
            if sys.modules.has_key(modname):
                globals = sys.modules[modname].__dict__
                self.globals = {}
            else:
                globals = self.globals
                modname = subname
        else:
            file = '<%s>' % self.title
            globals = self.globals
            modname = file
        return globals, file, modname

    def write(self, stuff):
        """for use as stdout"""
        self._buf = self._buf + stuff
        if '\n' in self._buf:
            self.flush()

    def flush(self):
        stuff = string.split(self._buf, '\n')
        stuff = string.join(stuff, '\r')
        end = self.editgroup.editor.ted.WEGetTextLength()
        self.editgroup.editor.ted.WESetSelection(end, end)
        self.editgroup.editor.ted.WEInsert(stuff, None, None)
        self.editgroup.editor.updatescrollbars()
        self._buf = ""
        # ? optional:
        #self.wid.SelectWindow()

    def getclasslist(self):
        from string import find, strip
        methodRE = re.compile(r"\r[ \t]+def ")
        findMethod = methodRE.search
        editor = self.editgroup.editor
        text = editor.get()
        list = []
        append = list.append
        functag = "func"
        classtag = "class"
        methodtag = "method"
        pos = -1
        if text[:4] == 'def ':
            append((pos + 4, functag))
            pos = 4
        while 1:
            pos = find(text, '\rdef ', pos + 1)
            if pos < 0:
                break
            append((pos + 5, functag))
        pos = -1
        if text[:6] == 'class ':
            append((pos + 6, classtag))
            pos = 6
        while 1:
            pos = find(text, '\rclass ', pos + 1)
            if pos < 0:
                break
            append((pos + 7, classtag))
        pos = 0
        while 1:
            m = findMethod(text, pos + 1)
            if m is None:
                break
            pos = m.regs[0][0]
            #pos = find(text, '\r\tdef ', pos + 1)
            append((m.regs[0][1], methodtag))
        list.sort()
        classlist = []
        methodlistappend = None
        offsetToLine = editor.ted.WEOffsetToLine
        getLineRange = editor.ted.WEGetLineRange
        append = classlist.append
        for pos, tag in list:
            lineno = offsetToLine(pos)
            lineStart, lineEnd = getLineRange(lineno)
            line = strip(text[pos:lineEnd])
            line = line[:identifieRE_match(line)]
            if tag is functag:
                append(("def " + line, lineno + 1))
                methodlistappend = None
            elif tag is classtag:
                append(["class " + line])
                methodlistappend = classlist[-1].append
            elif methodlistappend and tag is methodtag:
                methodlistappend(("def " + line, lineno + 1))
        return classlist

    def popselectline(self, lineno):
        self.editgroup.editor.selectline(lineno - 1)

    def selectline(self, lineno, charoffset = 0):
        self.editgroup.editor.selectline(lineno - 1, charoffset)

    def addrecentfile(self, filename):
        app = W.getapplication()
        app.addrecentfile(filename)

class _saveoptions:

    def __init__(self, creator, eoln):
        self.rv = None
        self.eoln = eoln
        self.w = w = W.ModalDialog((260, 160), 'Save options')
        radiobuttons = []
        w.label = W.TextBox((8, 8, 80, 18), "File creator:")
        w.ide_radio = W.RadioButton((8, 22, 160, 18), "PythonIDE", radiobuttons, self.ide_hit)
        w.interp_radio = W.RadioButton((8, 42, 160, 18), "MacPython-OS9 Interpreter", radiobuttons, self.interp_hit)
        w.interpx_radio = W.RadioButton((8, 62, 160, 18), "PythonLauncher", radiobuttons, self.interpx_hit)
        w.other_radio = W.RadioButton((8, 82, 50, 18), "Other:", radiobuttons)
        w.other_creator = W.EditText((62, 82, 40, 20), creator, self.otherselect)
        w.none_radio = W.RadioButton((8, 102, 160, 18), "None", radiobuttons, self.none_hit)
        w.cancelbutton = W.Button((-180, -30, 80, 16), "Cancel", self.cancelbuttonhit)
        w.okbutton = W.Button((-90, -30, 80, 16), "Done", self.okbuttonhit)
        w.setdefaultbutton(w.okbutton)
        if creator == 'Pyth':
            w.interp_radio.set(1)
        elif creator == W._signature:
            w.ide_radio.set(1)
        elif creator == 'PytX':
            w.interpx_radio.set(1)
        elif creator == '\0\0\0\0':
            w.none_radio.set(1)
        else:
            w.other_radio.set(1)

        w.eolnlabel = W.TextBox((168, 8, 80, 18), "Newline style:")
        radiobuttons = []
        w.unix_radio = W.RadioButton((168, 22, 80, 18), "Unix", radiobuttons, self.unix_hit)
        w.mac_radio = W.RadioButton((168, 42, 80, 18), "Macintosh", radiobuttons, self.mac_hit)
        w.win_radio = W.RadioButton((168, 62, 80, 18), "Windows", radiobuttons, self.win_hit)
        if self.eoln == '\n':
            w.unix_radio.set(1)
        elif self.eoln == '\r\n':
            w.win_radio.set(1)
        else:
            w.mac_radio.set(1)

        w.bind("cmd.", w.cancelbutton.push)
        w.open()

    def ide_hit(self):
        self.w.other_creator.set(W._signature)

    def interp_hit(self):
        self.w.other_creator.set("Pyth")

    def interpx_hit(self):
        self.w.other_creator.set("PytX")

    def none_hit(self):
        self.w.other_creator.set("\0\0\0\0")

    def otherselect(self, *args):
        sel_from, sel_to = self.w.other_creator.getselection()
        creator = self.w.other_creator.get()[:4]
        creator = creator + " " * (4 - len(creator))
        self.w.other_creator.set(creator)
        self.w.other_creator.setselection(sel_from, sel_to)
        self.w.other_radio.set(1)

    def mac_hit(self):
        self.eoln = '\r'

    def unix_hit(self):
        self.eoln = '\n'

    def win_hit(self):
        self.eoln = '\r\n'

    def cancelbuttonhit(self):
        self.w.close()

    def okbuttonhit(self):
        self.rv = (self.w.other_creator.get()[:4], self.eoln)
        self.w.close()


def SaveOptions(creator, eoln):
    s = _saveoptions(creator, eoln)
    return s.rv


def _escape(where, what) :
    return string.join(string.split(where, what), '\\' + what)

def _makewholewordpattern(word):
    # first, escape special regex chars
    for esc in "\\[]()|.*^+$?":
        word = _escape(word, esc)
    notwordcharspat = '[^' + _wordchars + ']'
    pattern = '(' + word + ')'
    if word[0] in _wordchars:
        pattern = notwordcharspat + pattern
    if word[-1] in _wordchars:
        pattern = pattern + notwordcharspat
    return re.compile(pattern)


class SearchEngine:

    def __init__(self):
        self.visible = 0
        self.w = None
        self.parms = {  "find": "",
                                "replace": "",
                                "wrap": 1,
                                "casesens": 1,
                                "wholeword": 1
                        }
        import MacPrefs
        prefs = MacPrefs.GetPrefs(W.getapplication().preffilepath)
        if prefs.searchengine:
            self.parms["casesens"] = prefs.searchengine.casesens
            self.parms["wrap"] = prefs.searchengine.wrap
            self.parms["wholeword"] = prefs.searchengine.wholeword

    def show(self):
        self.visible = 1
        if self.w:
            self.w.wid.ShowWindow()
            self.w.wid.SelectWindow()
            self.w.find.edit.select(1)
            self.w.find.edit.selectall()
            return
        self.w = W.Dialog((420, 150), "Find")

        self.w.find = TitledEditText((10, 4, 300, 36), "Search for:")
        self.w.replace = TitledEditText((10, 100, 300, 36), "Replace with:")

        self.w.boxes = W.Group((10, 50, 300, 40))
        self.w.boxes.casesens = W.CheckBox((0, 0, 100, 16), "Case sensitive")
        self.w.boxes.wholeword = W.CheckBox((0, 20, 100, 16), "Whole word")
        self.w.boxes.wrap = W.CheckBox((110, 0, 100, 16), "Wrap around")

        self.buttons = [        ("Find",                "cmdf",  self.find),
                                ("Replace",          "cmdr",     self.replace),
                                ("Replace all",  None,   self.replaceall),
                                ("Don't find",  "cmdd",  self.dont),
                                ("Cancel",            "cmd.",    self.cancel)
                        ]
        for i in range(len(self.buttons)):
            bounds = -90, 22 + i * 24, 80, 16
            title, shortcut, callback = self.buttons[i]
            self.w[title] = W.Button(bounds, title, callback)
            if shortcut:
                self.w.bind(shortcut, self.w[title].push)
        self.w.setdefaultbutton(self.w["Don't find"])
        self.w.find.edit.bind("<key>", self.key)
        self.w.bind("<activate>", self.activate)
        self.w.bind("<close>", self.close)
        self.w.open()
        self.setparms()
        self.w.find.edit.select(1)
        self.w.find.edit.selectall()
        self.checkbuttons()

    def close(self):
        self.hide()
        return -1

    def key(self, char, modifiers):
        self.w.find.edit.key(char, modifiers)
        self.checkbuttons()
        return 1

    def activate(self, onoff):
        if onoff:
            self.checkbuttons()

    def checkbuttons(self):
        editor = findeditor(self)
        if editor:
            if self.w.find.get():
                for title, cmd, call in self.buttons[:-2]:
                    self.w[title].enable(1)
                self.w.setdefaultbutton(self.w["Find"])
            else:
                for title, cmd, call in self.buttons[:-2]:
                    self.w[title].enable(0)
                self.w.setdefaultbutton(self.w["Don't find"])
        else:
            for title, cmd, call in self.buttons[:-2]:
                self.w[title].enable(0)
            self.w.setdefaultbutton(self.w["Don't find"])

    def find(self):
        self.getparmsfromwindow()
        if self.findnext():
            self.hide()

    def replace(self):
        editor = findeditor(self)
        if not editor:
            return
        if self.visible:
            self.getparmsfromwindow()
        text = editor.getselectedtext()
        find = self.parms["find"]
        if not self.parms["casesens"]:
            find = string.lower(find)
            text = string.lower(text)
        if text == find:
            self.hide()
            editor.insert(self.parms["replace"])

    def replaceall(self):
        editor = findeditor(self)
        if not editor:
            return
        if self.visible:
            self.getparmsfromwindow()
        W.SetCursor("watch")
        find = self.parms["find"]
        if not find:
            return
        findlen = len(find)
        replace = self.parms["replace"]
        replacelen = len(replace)
        Text = editor.get()
        if not self.parms["casesens"]:
            find = string.lower(find)
            text = string.lower(Text)
        else:
            text = Text
        newtext = ""
        pos = 0
        counter = 0
        while 1:
            if self.parms["wholeword"]:
                wholewordRE = _makewholewordpattern(find)
                match = wholewordRE.search(text, pos)
                if match:
                    pos = match.start(1)
                else:
                    pos = -1
            else:
                pos = string.find(text, find, pos)
            if pos < 0:
                break
            counter = counter + 1
            text = text[:pos] + replace + text[pos + findlen:]
            Text = Text[:pos] + replace + Text[pos + findlen:]
            pos = pos + replacelen
        W.SetCursor("arrow")
        if counter:
            self.hide()
            from Carbon import Res
            editor.textchanged()
            editor.selectionchanged()
            editor.set(Text)
            EasyDialogs.Message("Replaced %d occurrences" % counter)

    def dont(self):
        self.getparmsfromwindow()
        self.hide()

    def replacefind(self):
        self.replace()
        self.findnext()

    def setfindstring(self):
        editor = findeditor(self)
        if not editor:
            return
        find = editor.getselectedtext()
        if not find:
            return
        self.parms["find"] = find
        if self.w:
            self.w.find.edit.set(self.parms["find"])
            self.w.find.edit.selectall()

    def findnext(self):
        editor = findeditor(self)
        if not editor:
            return
        find = self.parms["find"]
        if not find:
            return
        text = editor.get()
        if not self.parms["casesens"]:
            find = string.lower(find)
            text = string.lower(text)
        selstart, selend = editor.getselection()
        selstart, selend = min(selstart, selend), max(selstart, selend)
        if self.parms["wholeword"]:
            wholewordRE = _makewholewordpattern(find)
            match = wholewordRE.search(text, selend)
            if match:
                pos = match.start(1)
            else:
                pos = -1
        else:
            pos = string.find(text, find, selend)
        if pos >= 0:
            editor.setselection(pos, pos + len(find))
            return 1
        elif self.parms["wrap"]:
            if self.parms["wholeword"]:
                match = wholewordRE.search(text, 0)
                if match:
                    pos = match.start(1)
                else:
                    pos = -1
            else:
                pos = string.find(text, find)
            if selstart > pos >= 0:
                editor.setselection(pos, pos + len(find))
                return 1

    def setparms(self):
        for key, value in self.parms.items():
            try:
                self.w[key].set(value)
            except KeyError:
                self.w.boxes[key].set(value)

    def getparmsfromwindow(self):
        if not self.w:
            return
        for key, value in self.parms.items():
            try:
                value = self.w[key].get()
            except KeyError:
                value = self.w.boxes[key].get()
            self.parms[key] = value

    def cancel(self):
        self.hide()
        self.setparms()

    def hide(self):
        if self.w:
            self.w.wid.HideWindow()
            self.visible = 0

    def writeprefs(self):
        import MacPrefs
        self.getparmsfromwindow()
        prefs = MacPrefs.GetPrefs(W.getapplication().preffilepath)
        prefs.searchengine.casesens = self.parms["casesens"]
        prefs.searchengine.wrap = self.parms["wrap"]
        prefs.searchengine.wholeword = self.parms["wholeword"]
        prefs.save()


class TitledEditText(W.Group):

    def __init__(self, possize, title, text = ""):
        W.Group.__init__(self, possize)
        self.title = W.TextBox((0, 0, 0, 16), title)
        self.edit = W.EditText((0, 16, 0, 0), text)

    def set(self, value):
        self.edit.set(value)

    def get(self):
        return self.edit.get()


class ClassFinder(W.PopupWidget):

    def click(self, point, modifiers):
        W.SetCursor("watch")
        self.set(self._parentwindow.getclasslist())
        W.PopupWidget.click(self, point, modifiers)


def getminindent(lines):
    indent = -1
    for line in lines:
        stripped = string.strip(line)
        if not stripped or stripped[0] == '#':
            continue
        if indent < 0 or line[:indent] <> indent * '\t':
            indent = 0
            for c in line:
                if c <> '\t':
                    break
                indent = indent + 1
    return indent


def getoptionkey():
    return not not ord(Evt.GetKeys()[7]) & 0x04


def execstring(pytext, globals, locals, filename="<string>", debugging=0,
                        modname="__main__", profiling=0):
    if debugging:
        import PyDebugger, bdb
        BdbQuit = bdb.BdbQuit
    else:
        BdbQuit = 'BdbQuitDummyException'
    pytext = string.split(pytext, '\r')
    pytext = string.join(pytext, '\n') + '\n'
    W.SetCursor("watch")
    globals['__name__'] = modname
    globals['__file__'] = filename
    sys.argv = [filename]
    try:
        code = compile(pytext, filename, "exec")
    except:
        # XXXX BAAAADDD.... We let tracebackwindow decide to treat SyntaxError
        # special. That's wrong because THIS case is special (could be literal
        # overflow!) and SyntaxError could mean we need a traceback (syntax error
        # in imported module!!!
        tracebackwindow.traceback(1, filename)
        return
    try:
        if debugging:
            PyDebugger.startfromhere()
        else:
            if hasattr(MacOS, 'EnableAppswitch'):
                MacOS.EnableAppswitch(0)
        try:
            if profiling:
                import profile, ProfileBrowser
                p = profile.Profile()
                p.set_cmd(filename)
                try:
                    p.runctx(code, globals, locals)
                finally:
                    import pstats

                    stats = pstats.Stats(p)
                    ProfileBrowser.ProfileBrowser(stats)
            else:
                exec code in globals, locals
        finally:
            if hasattr(MacOS, 'EnableAppswitch'):
                MacOS.EnableAppswitch(-1)
    except W.AlertError, detail:
        raise W.AlertError, detail
    except (KeyboardInterrupt, BdbQuit):
        pass
    except SystemExit, arg:
        if arg.code:
            sys.stderr.write("Script exited with status code: %s\n" % repr(arg.code))
    except:
        if debugging:
            sys.settrace(None)
            PyDebugger.postmortem(sys.exc_type, sys.exc_value, sys.exc_traceback)
            return
        else:
            tracebackwindow.traceback(1, filename)
    if debugging:
        sys.settrace(None)
        PyDebugger.stop()


_identifieRE = re.compile(r"[A-Za-z_][A-Za-z_0-9]*")

def identifieRE_match(str):
    match = _identifieRE.match(str)
    if not match:
        return -1
    return match.end()

def _filename_as_modname(fname):
    if fname[-3:] == '.py':
        modname = fname[:-3]
        match = _identifieRE.match(modname)
        if match and match.start() == 0 and match.end() == len(modname):
            return string.join(string.split(modname, '.'), '_')

def findeditor(topwindow, fromtop = 0):
    wid = MyFrontWindow()
    if not fromtop:
        if topwindow.w and wid == topwindow.w.wid:
            wid = topwindow.w.wid.GetNextWindow()
    if not wid:
        return
    app = W.getapplication()
    if app._windows.has_key(wid): # KeyError otherwise can happen in RoboFog :-(
        window = W.getapplication()._windows[wid]
    else:
        return
    if not isinstance(window, Editor):
        return
    return window.editgroup.editor


class _EditorDefaultSettings:

    def __init__(self):
        self.template = "%s, %d point"
        self.fontsettings, self.tabsettings, self.windowsize = geteditorprefs()
        self.w = W.Dialog((328, 120), "Editor default settings")
        self.w.setfontbutton = W.Button((8, 8, 80, 16), "Set font\xc9", self.dofont)
        self.w.fonttext = W.TextBox((98, 10, -8, 14), self.template % (self.fontsettings[0], self.fontsettings[2]))

        self.w.picksizebutton = W.Button((8, 50, 80, 16), "Front window", self.picksize)
        self.w.xsizelabel = W.TextBox((98, 32, 40, 14), "Width:")
        self.w.ysizelabel = W.TextBox((148, 32, 40, 14), "Height:")
        self.w.xsize = W.EditText((98, 48, 40, 20), repr(self.windowsize[0]))
        self.w.ysize = W.EditText((148, 48, 40, 20), repr(self.windowsize[1]))

        self.w.cancelbutton = W.Button((-180, -26, 80, 16), "Cancel", self.cancel)
        self.w.okbutton = W.Button((-90, -26, 80, 16), "Done", self.ok)
        self.w.setdefaultbutton(self.w.okbutton)
        self.w.bind('cmd.', self.w.cancelbutton.push)
        self.w.open()

    def picksize(self):
        app = W.getapplication()
        editor = findeditor(self)
        if editor is not None:
            width, height = editor._parentwindow._bounds[2:]
            self.w.xsize.set(repr(width))
            self.w.ysize.set(repr(height))
        else:
            raise W.AlertError, "No edit window found"

    def dofont(self):
        import FontSettings
        settings = FontSettings.FontDialog(self.fontsettings, self.tabsettings)
        if settings:
            self.fontsettings, self.tabsettings = settings
            sys.exc_traceback = None
            self.w.fonttext.set(self.template % (self.fontsettings[0], self.fontsettings[2]))

    def close(self):
        self.w.close()
        del self.w

    def cancel(self):
        self.close()

    def ok(self):
        try:
            width = string.atoi(self.w.xsize.get())
        except:
            self.w.xsize.select(1)
            self.w.xsize.selectall()
            raise W.AlertError, "Bad number for window width"
        try:
            height = string.atoi(self.w.ysize.get())
        except:
            self.w.ysize.select(1)
            self.w.ysize.selectall()
            raise W.AlertError, "Bad number for window height"
        self.windowsize = width, height
        seteditorprefs(self.fontsettings, self.tabsettings, self.windowsize)
        self.close()

def geteditorprefs():
    import MacPrefs
    prefs = MacPrefs.GetPrefs(W.getapplication().preffilepath)
    try:
        fontsettings = prefs.pyedit.fontsettings
        tabsettings = prefs.pyedit.tabsettings
        windowsize = prefs.pyedit.windowsize
    except:
        fontsettings = prefs.pyedit.fontsettings = ("Geneva", 0, 10, (0, 0, 0))
        tabsettings = prefs.pyedit.tabsettings = (8, 1)
        windowsize = prefs.pyedit.windowsize = (500, 250)
        sys.exc_traceback = None
    return fontsettings, tabsettings, windowsize

def seteditorprefs(fontsettings, tabsettings, windowsize):
    import MacPrefs
    prefs = MacPrefs.GetPrefs(W.getapplication().preffilepath)
    prefs.pyedit.fontsettings = fontsettings
    prefs.pyedit.tabsettings = tabsettings
    prefs.pyedit.windowsize = windowsize
    prefs.save()

_defaultSettingsEditor = None

def EditorDefaultSettings():
    global _defaultSettingsEditor
    if _defaultSettingsEditor is None or not hasattr(_defaultSettingsEditor, "w"):
        _defaultSettingsEditor = _EditorDefaultSettings()
    else:
        _defaultSettingsEditor.w.select()

def resolvealiases(path):
    try:
        fsr, d1, d2 = File.FSResolveAliasFile(path, 1)
        path = fsr.as_pathname()
        return path
    except (File.Error, ValueError), (error, str):
        if error <> -120:
            raise
        dir, file = os.path.split(path)
        return os.path.join(resolvealiases(dir), file)

searchengine = SearchEngine()
tracebackwindow = Wtraceback.TraceBack()



# This script generates a Python interface for an Apple Macintosh Manager.
# It uses the "bgen" package to generate C code.
# The function specifications are generated by scanning the mamager's header file,
# using the "scantools" package (customized for this particular manager).

import string

# Declarations that change for each manager
MODNAME = '_Launch'                             # The name of the module
OBJECTNAME = 'UNUSED'                   # The basic name of the objects used here
KIND = 'Record'                         # Usually 'Ptr' or 'Handle'

# The following is *usually* unchanged but may still require tuning
MODPREFIX = 'Launch'                    # The prefix for module-wide routines
OBJECTTYPE = OBJECTNAME + KIND          # The C type used to represent them
OBJECTPREFIX = MODPREFIX + 'Obj'        # The prefix for object methods
INPUTFILE = string.lower(MODPREFIX) + 'gen.py' # The file generated by the scanner
OUTPUTFILE = MODNAME + "module.c"       # The file generated by this program

from macsupport import *

# Create the type objects
LSAcceptanceFlags = Type("LSAcceptanceFlags", "l")
LSInitializeFlags = Type("LSInitializeFlags", "l")
LSRequestedInfo = Type("LSRequestedInfo", "l")
LSRolesMask = Type("LSRolesMask", "l")
UniCharCount = Type("UniCharCount", "l")
OptCFStringRef = OpaqueByValueType("CFStringRef", "OptCFStringRefObj")
LSItemInfoRecord = OpaqueType("LSItemInfoRecord", "LSItemInfoRecord")

includestuff = includestuff + """
#if PY_VERSION_HEX < 0x02040000
PyObject *PyMac_GetOSErrException(void);
#endif

#include <ApplicationServices/ApplicationServices.h>

/*
** Optional CFStringRef. None will pass NULL
*/
static int
OptCFStringRefObj_Convert(PyObject *v, CFStringRef *spec)
{
        if (v == Py_None) {
                *spec = NULL;
                return 1;
        }
        return CFStringRefObj_Convert(v, spec);
}

PyObject *
OptCFStringRefObj_New(CFStringRef it)
{
        if (it == NULL) {
                Py_INCREF(Py_None);
                return Py_None;
        }
        return CFStringRefObj_New(it);
}

/*
** Convert LSItemInfoRecord to Python.
*/
PyObject *
LSItemInfoRecord_New(LSItemInfoRecord *it)
{
        return Py_BuildValue("{s:is:O&s:O&s:O&s:O&s:i}",
                "flags", it->flags,
                "filetype", PyMac_BuildOSType, it->filetype,
                "creator", PyMac_BuildOSType, it->creator,
                "extension", OptCFStringRefObj_New, it->extension,
                "iconFileName", OptCFStringRefObj_New, it->iconFileName,
                "kindID", it->kindID);
}
"""

# From here on it's basically all boiler plate...
execfile(string.lower(MODPREFIX) + 'typetest.py')

# Create the generator groups and link them
module = MacModule(MODNAME, MODPREFIX, includestuff, finalstuff, initstuff)
##object = MyObjectDefinition(OBJECTNAME, OBJECTPREFIX, OBJECTTYPE)
##module.addobject(object)

# Create the generator classes used to populate the lists
Function = OSErrFunctionGenerator
##Method = OSErrMethodGenerator

# Create and populate the lists
functions = []
##methods = []
execfile(INPUTFILE)

# add the populated lists to the generator groups
# (in a different wordl the scan program would generate this)
for f in functions: module.add(f)
##for f in methods: object.add(f)

# generate output (open the output file as late as possible)
SetOutputFileName(OUTPUTFILE)
module.generate()



"""macgen_info - Generate informational output"""

def generate(output, module_dict):
    for name in module_dict.keys():
        print 'Include %-20s\t'%name,
        module = module_dict[name]
        print module.gettype(), '\t', repr(module)
    return 0



import os

# This module should be kept compatible with Python 1.5.2.

__revision__ = "$Id: debug.py,v 1.3 2004/07/18 06:14:42 tim_one Exp $"

# If DISTUTILS_DEBUG is anything other than the empty string, we run in
# debug mode.
DEBUG = os.environ.get('DISTUTILS_DEBUG')



# New dir() function


# This should be the new dir(), except that it should still list
# the current local name space by default

def listattrs(x):
    try:
        dictkeys = x.__dict__.keys()
    except (AttributeError, TypeError):
        dictkeys = []
    #
    try:
        methods = x.__methods__
    except (AttributeError, TypeError):
        methods = []
    #
    try:
        members = x.__members__
    except (AttributeError, TypeError):
        members = []
    #
    try:
        the_class = x.__class__
    except (AttributeError, TypeError):
        the_class = None
    #
    try:
        bases = x.__bases__
    except (AttributeError, TypeError):
        bases = ()
    #
    total = dictkeys + methods + members
    if the_class:
        # It's a class instace; add the class's attributes
        # that are functions (methods)...
        class_attrs = listattrs(the_class)
        class_methods = []
        for name in class_attrs:
            if is_function(getattr(the_class, name)):
                class_methods.append(name)
        total = total + class_methods
    elif bases:
        # It's a derived class; add the base class attributes
        for base in bases:
            base_attrs = listattrs(base)
            total = total + base_attrs
    total.sort()
    return total
    i = 0
    while i+1 < len(total):
        if total[i] == total[i+1]:
            del total[i+1]
        else:
            i = i+1
    return total


# Helper to recognize functions

def is_function(x):
    return type(x) == type(is_function)


# Approximation of builtin dir(); but note that this lists the user's
# variables by default, not the current local name space.

def dir(x = None):
    if x is not None:
        return listattrs(x)
    else:
        import __main__
        return listattrs(__main__)



"""Generic (shallow and deep) copying operations.

Interface summary:

        import copy

        x = copy.copy(y)        # make a shallow copy of y
        x = copy.deepcopy(y)    # make a deep copy of y

For module specific errors, copy.Error is raised.

The difference between shallow and deep copying is only relevant for
compound objects (objects that contain other objects, like lists or
class instances).

- A shallow copy constructs a new compound object and then (to the
  extent possible) inserts *the same objects* into in that the
  original contains.

- A deep copy constructs a new compound object and then, recursively,
  inserts *copies* into it of the objects found in the original.

Two problems often exist with deep copy operations that don't exist
with shallow copy operations:

 a) recursive objects (compound objects that, directly or indirectly,
    contain a reference to themselves) may cause a recursive loop

 b) because deep copy copies *everything* it may copy too much, e.g.
    administrative data structures that should be shared even between
    copies

Python's deep copy operation avoids these problems by:

 a) keeping a table of objects already copied during the current
    copying pass

 b) letting user-defined classes override the copying operation or the
    set of components copied

This version does not copy types like module, class, function, method,
nor stack trace, stack frame, nor file, socket, window, nor array, nor
any similar types.

Classes can use the same interfaces to control copying that they use
to control pickling: they can define methods called __getinitargs__(),
__getstate__() and __setstate__().  See the documentation for module
"pickle" for information on these methods.
"""

import types
from copy_reg import dispatch_table

class Error(Exception):
    pass
error = Error   # backward compatibility

try:
    from org.python.core import PyStringMap
except ImportError:
    PyStringMap = None

__all__ = ["Error", "copy", "deepcopy"]

def copy(x):
    """Shallow copy operation on arbitrary Python objects.

    See the module's __doc__ string for more info.
    """

    cls = type(x)

    copier = _copy_dispatch.get(cls)
    if copier:
        return copier(x)

    copier = getattr(cls, "__copy__", None)
    if copier:
        return copier(x)

    reductor = dispatch_table.get(cls)
    if reductor:
        rv = reductor(x)
    else:
        reductor = getattr(x, "__reduce_ex__", None)
        if reductor:
            rv = reductor(2)
        else:
            reductor = getattr(x, "__reduce__", None)
            if reductor:
                rv = reductor()
            else:
                raise Error("un(shallow)copyable object of type %s" % cls)

    return _reconstruct(x, rv, 0)


_copy_dispatch = d = {}

def _copy_immutable(x):
    return x
for t in (types.NoneType, int, long, float, bool, str, tuple,
          frozenset, type, xrange, types.ClassType,
          types.BuiltinFunctionType):
    d[t] = _copy_immutable
for name in ("ComplexType", "UnicodeType", "CodeType"):
    t = getattr(types, name, None)
    if t is not None:
        d[t] = _copy_immutable

def _copy_with_constructor(x):
    return type(x)(x)
for t in (list, dict, set):
    d[t] = _copy_with_constructor

def _copy_with_copy_method(x):
    return x.copy()
if PyStringMap is not None:
    d[PyStringMap] = _copy_with_copy_method

def _copy_inst(x):
    if hasattr(x, '__copy__'):
        return x.__copy__()
    if hasattr(x, '__getinitargs__'):
        args = x.__getinitargs__()
        y = x.__class__(*args)
    else:
        y = _EmptyClass()
        y.__class__ = x.__class__
    if hasattr(x, '__getstate__'):
        state = x.__getstate__()
    else:
        state = x.__dict__
    if hasattr(y, '__setstate__'):
        y.__setstate__(state)
    else:
        y.__dict__.update(state)
    return y
d[types.InstanceType] = _copy_inst

del d

def deepcopy(x, memo=None, _nil=[]):
    """Deep copy operation on arbitrary Python objects.

    See the module's __doc__ string for more info.
    """

    if memo is None:
        memo = {}

    d = id(x)
    y = memo.get(d, _nil)
    if y is not _nil:
        return y

    cls = type(x)

    copier = _deepcopy_dispatch.get(cls)
    if copier:
        y = copier(x, memo)
    else:
        try:
            issc = issubclass(cls, type)
        except TypeError: # cls is not a class (old Boost; see SF #502085)
            issc = 0
        if issc:
            y = _deepcopy_atomic(x, memo)
        else:
            copier = getattr(x, "__deepcopy__", None)
            if copier:
                y = copier(memo)
            else:
                reductor = dispatch_table.get(cls)
                if reductor:
                    rv = reductor(x)
                else:
                    reductor = getattr(x, "__reduce_ex__", None)
                    if reductor:
                        rv = reductor(2)
                    else:
                        reductor = getattr(x, "__reduce__", None)
                        if reductor:
                            rv = reductor()
                        else:
                            raise Error(
                                "un(deep)copyable object of type %s" % cls)
                y = _reconstruct(x, rv, 1, memo)

    memo[d] = y
    _keep_alive(x, memo) # Make sure x lives at least as long as d
    return y

_deepcopy_dispatch = d = {}

def _deepcopy_atomic(x, memo):
    return x
d[types.NoneType] = _deepcopy_atomic
d[types.IntType] = _deepcopy_atomic
d[types.LongType] = _deepcopy_atomic
d[types.FloatType] = _deepcopy_atomic
d[types.BooleanType] = _deepcopy_atomic
try:
    d[types.ComplexType] = _deepcopy_atomic
except AttributeError:
    pass
d[types.StringType] = _deepcopy_atomic
try:
    d[types.UnicodeType] = _deepcopy_atomic
except AttributeError:
    pass
try:
    d[types.CodeType] = _deepcopy_atomic
except AttributeError:
    pass
d[types.TypeType] = _deepcopy_atomic
d[types.XRangeType] = _deepcopy_atomic
d[types.ClassType] = _deepcopy_atomic
d[types.BuiltinFunctionType] = _deepcopy_atomic

def _deepcopy_list(x, memo):
    y = []
    memo[id(x)] = y
    for a in x:
        y.append(deepcopy(a, memo))
    return y
d[types.ListType] = _deepcopy_list

def _deepcopy_tuple(x, memo):
    y = []
    for a in x:
        y.append(deepcopy(a, memo))
    d = id(x)
    try:
        return memo[d]
    except KeyError:
        pass
    for i in range(len(x)):
        if x[i] is not y[i]:
            y = tuple(y)
            break
    else:
        y = x
    memo[d] = y
    return y
d[types.TupleType] = _deepcopy_tuple

def _deepcopy_dict(x, memo):
    y = {}
    memo[id(x)] = y
    for key, value in x.iteritems():
        y[deepcopy(key, memo)] = deepcopy(value, memo)
    return y
d[types.DictionaryType] = _deepcopy_dict
if PyStringMap is not None:
    d[PyStringMap] = _deepcopy_dict

def _keep_alive(x, memo):
    """Keeps a reference to the object x in the memo.

    Because we remember objects by their id, we have
    to assure that possibly temporary objects are kept
    alive by referencing them.
    We store a reference at the id of the memo, which should
    normally not be used unless someone tries to deepcopy
    the memo itself...
    """
    try:
        memo[id(memo)].append(x)
    except KeyError:
        # aha, this is the first one :-)
        memo[id(memo)]=[x]

def _deepcopy_inst(x, memo):
    if hasattr(x, '__deepcopy__'):
        return x.__deepcopy__(memo)
    if hasattr(x, '__getinitargs__'):
        args = x.__getinitargs__()
        args = deepcopy(args, memo)
        y = x.__class__(*args)
    else:
        y = _EmptyClass()
        y.__class__ = x.__class__
    memo[id(x)] = y
    if hasattr(x, '__getstate__'):
        state = x.__getstate__()
    else:
        state = x.__dict__
    state = deepcopy(state, memo)
    if hasattr(y, '__setstate__'):
        y.__setstate__(state)
    else:
        y.__dict__.update(state)
    return y
d[types.InstanceType] = _deepcopy_inst

def _reconstruct(x, info, deep, memo=None):
    if isinstance(info, str):
        return x
    assert isinstance(info, tuple)
    if memo is None:
        memo = {}
    n = len(info)
    assert n in (2, 3, 4, 5)
    callable, args = info[:2]
    if n > 2:
        state = info[2]
    else:
        state = {}
    if n > 3:
        listiter = info[3]
    else:
        listiter = None
    if n > 4:
        dictiter = info[4]
    else:
        dictiter = None
    if deep:
        args = deepcopy(args, memo)
    y = callable(*args)
    memo[id(x)] = y
    if listiter is not None:
        for item in listiter:
            if deep:
                item = deepcopy(item, memo)
            y.append(item)
    if dictiter is not None:
        for key, value in dictiter:
            if deep:
                key = deepcopy(key, memo)
                value = deepcopy(value, memo)
            y[key] = value
    if state:
        if deep:
            state = deepcopy(state, memo)
        if hasattr(y, '__setstate__'):
            y.__setstate__(state)
        else:
            if isinstance(state, tuple) and len(state) == 2:
                state, slotstate = state
            else:
                slotstate = None
            if state is not None:
                y.__dict__.update(state)
            if slotstate is not None:
                for key, value in slotstate.iteritems():
                    setattr(y, key, value)
    return y

del d

del types

# Helper for instance creation without calling __init__
class _EmptyClass:
    pass

def _test():
    l = [None, 1, 2L, 3.14, 'xyzzy', (1, 2L), [3.14, 'abc'],
         {'abc': 'ABC'}, (), [], {}]
    l1 = copy(l)
    print l1==l
    l1 = map(copy, l)
    print l1==l
    l1 = deepcopy(l)
    print l1==l
    class C:
        def __init__(self, arg=None):
            self.a = 1
            self.arg = arg
            if __name__ == '__main__':
                import sys
                file = sys.argv[0]
            else:
                file = __file__
            self.fp = open(file)
            self.fp.close()
        def __getstate__(self):
            return {'a': self.a, 'arg': self.arg}
        def __setstate__(self, state):
            for key, value in state.iteritems():
                setattr(self, key, value)
        def __deepcopy__(self, memo=None):
            new = self.__class__(deepcopy(self.arg, memo))
            new.a = self.a
            return new
    c = C('argument sketch')
    l.append(c)
    l2 = copy(l)
    print l == l2
    print l
    print l2
    l2 = deepcopy(l)
    print l == l2
    print l
    print l2
    l.append({l[1]: l, 'xyz': l[2]})
    l3 = copy(l)
    import repr
    print map(repr.repr, l)
    print map(repr.repr, l1)
    print map(repr.repr, l2)
    print map(repr.repr, l3)
    l3 = deepcopy(l)
    import repr
    print map(repr.repr, l)
    print map(repr.repr, l1)
    print map(repr.repr, l2)
    print map(repr.repr, l3)

if __name__ == '__main__':
    _test()



"""checktext - Check that a text file has macintosh-style newlines"""

import sys
import EasyDialogs
import string

def main():
    pathname = EasyDialogs.AskFileForOpen(message='File to check end-of-lines in:')
    if not pathname:
        sys.exit(0)
    fp = open(pathname, 'rb')
    try:
        data = fp.read()
    except MemoryError:
        EasyDialogs.Message('Sorry, file is too big.')
        sys.exit(0)
    if len(data) == 0:
        EasyDialogs.Message('File is empty.')
        sys.exit(0)
    number_cr = string.count(data, '\r')
    number_lf = string.count(data, '\n')
    if number_cr == number_lf == 0:
        EasyDialogs.Message('File contains no lines.')
    if number_cr == 0:
        EasyDialogs.Message('File has unix-style line endings')
    elif number_lf == 0:
        EasyDialogs.Message('File has mac-style line endings')
    elif number_cr == number_lf:
        EasyDialogs.Message('File probably has MSDOS-style line endings')
    else:
        EasyDialogs.Message('File has no recognizable line endings (binary file?)')
    sys.exit(0)

if __name__ == '__main__':
    main()



#! /usr/bin/env python

# This Python program sorts and reformats the table of keywords in ref2.tex

l = []
try:
    while 1:
        l = l + raw_input().split()
except EOFError:
    pass
l.sort()
for x in l[:]:
    while l.count(x) > 1: l.remove(x)
ncols = 5
nrows = (len(l)+ncols-1)/ncols
for i in range(nrows):
    for j in range(i, len(l), nrows):
        print l[j].ljust(10),
    print



# A minimal text editor.
#
# To be done:
# - Functionality: find, etc.

from Carbon.Menu import DrawMenuBar
from FrameWork import *
from Carbon import Win
from Carbon import Qd
from Carbon import Res
import waste
import WASTEconst
from Carbon import Scrap
import os
import EasyDialogs

UNDOLABELS = [ # Indexed by WEGetUndoInfo() value
        None, "", "typing", "Cut", "Paste", "Clear", "Drag", "Style"]

BIGREGION=Qd.NewRgn()
Qd.SetRectRgn(BIGREGION, -16000, -16000, 16000, 16000)

class WasteWindow(ScrolledWindow):
    def open(self, path, name, data):
        self.path = path
        self.name = name
        r = windowbounds(400, 400)
        w = Win.NewWindow(r, name, 1, 0, -1, 1, 0)
        self.wid = w
        vr = 0, 0, r[2]-r[0]-15, r[3]-r[1]-15
        dr = (0, 0, 10240, 0)
        Qd.SetPort(w)
        Qd.TextFont(4)
        Qd.TextSize(9)
        flags = WASTEconst.weDoAutoScroll | WASTEconst.weDoOutlineHilite | \
                WASTEconst.weDoMonoStyled | WASTEconst.weDoUndo
        self.ted = waste.WENew(dr, vr, flags)
        self.tedtexthandle = Res.Resource(data)
        self.ted.WEUseText(self.tedtexthandle)
        self.ted.WECalText()
        w.DrawGrowIcon()
        self.scrollbars()
        self.changed = 0
        self.do_postopen()
        self.do_activate(1, None)

    def do_idle(self, event):
        (what, message, when, where, modifiers) = event
        Qd.SetPort(self.wid)
        self.ted.WEIdle()
        if self.ted.WEAdjustCursor(where, BIGREGION):
            return
        Qd.SetCursor(Qd.GetQDGlobalsArrow())

    def getscrollbarvalues(self):
        dr = self.ted.WEGetDestRect()
        vr = self.ted.WEGetViewRect()
        vx = self.scalebarvalue(dr[0], dr[2], vr[0], vr[2])
        vy = self.scalebarvalue(dr[1], dr[3], vr[1], vr[3])
##              print dr, vr, vx, vy
        return vx, vy

    def scrollbar_callback(self, which, what, value):
        if which == 'y':
            if what == 'set':
                height = self.ted.WEGetHeight(0, 0x3fffffff)
                cur = self.getscrollbarvalues()[1]
                delta = (cur-value)*height/32767
            if what == '-':
                topline_off,dummy = self.ted.WEGetOffset((1,1))
                topline_num = self.ted.WEOffsetToLine(topline_off)
                delta = self.ted.WEGetHeight(topline_num, topline_num+1)
            elif what == '--':
                delta = (self.ted.WEGetViewRect()[3]-10)
                if delta <= 0:
                    delta = 10 # Random value
            elif what == '+':
                # XXXX Wrong: should be bottom line size
                topline_off,dummy = self.ted.WEGetOffset((1,1))
                topline_num = self.ted.WEOffsetToLine(topline_off)
                delta = -self.ted.WEGetHeight(topline_num, topline_num+1)
            elif what == '++':
                delta = -(self.ted.WEGetViewRect()[3]-10)
                if delta >= 0:
                    delta = -10
            self.ted.WEScroll(0, delta)
##                      print 'SCROLL Y', delta
        else:
            if what == 'set':
                return # XXXX
            vr = self.ted.WEGetViewRect()
            winwidth = vr[2]-vr[0]
            if what == '-':
                delta = winwidth/10
            elif what == '--':
                delta = winwidth/2
            elif what == '+':
                delta = -winwidth/10
            elif what == '++':
                delta = -winwidth/2
            self.ted.WEScroll(delta, 0)
        # Pin the scroll
        l, t, r, b = self.ted.WEGetDestRect()
        vl, vt, vr, vb = self.ted.WEGetViewRect()
        if t > 0 or l > 0:
            dx = dy = 0
            if t > 0: dy = -t
            if l > 0: dx = -l
##                      print 'Extra scroll', dx, dy
            self.ted.WEScroll(dx, dy)
        elif b < vb:
##                      print 'Extra downscroll', b-vb
            self.ted.WEScroll(0, b-vb)


    def do_activate(self, onoff, evt):
##              print "ACTIVATE", onoff
        Qd.SetPort(self.wid)
        ScrolledWindow.do_activate(self, onoff, evt)
        if onoff:
            self.ted.WEActivate()
            self.parent.active = self
            self.parent.updatemenubar()
        else:
            self.ted.WEDeactivate()

    def do_update(self, wid, event):
        region = wid.GetWindowPort().visRgn
        if Qd.EmptyRgn(region):
            return
        Qd.EraseRgn(region)
        self.ted.WEUpdate(region)
        self.updatescrollbars()

    def do_postresize(self, width, height, window):
        l, t, r, b = self.ted.WEGetViewRect()
        vr = (l, t, l+width-15, t+height-15)
        self.ted.WESetViewRect(vr)
        self.wid.InvalWindowRect(vr)
        ScrolledWindow.do_postresize(self, width, height, window)

    def do_contentclick(self, local, modifiers, evt):
        (what, message, when, where, modifiers) = evt
        self.ted.WEClick(local, modifiers, when)
        self.updatescrollbars()
        self.parent.updatemenubar()

    def do_char(self, ch, event):
        self.ted.WESelView()
        (what, message, when, where, modifiers) = event
        self.ted.WEKey(ord(ch), modifiers)
        self.changed = 1
        self.updatescrollbars()
        self.parent.updatemenubar()

    def close(self):
        if self.changed:
            save = EasyDialogs.AskYesNoCancel('Save window "%s" before closing?'%self.name, 1)
            if save > 0:
                self.menu_save()
            elif save < 0:
                return
        if self.parent.active == self:
            self.parent.active = None
        self.parent.updatemenubar()
        del self.ted
        del self.tedtexthandle
        self.do_postclose()

    def menu_save(self):
        if not self.path:
            self.menu_save_as()
            return # Will call us recursively
##              print 'Saving to ', self.path
        dhandle = self.ted.WEGetText()
        data = dhandle.data
        fp = open(self.path, 'wb')  # NOTE: wb, because data has CR for end-of-line
        fp.write(data)
        if data[-1] <> '\r': fp.write('\r')
        fp.close()
        self.changed = 0

    def menu_save_as(self):
        path = EasyDialogs.AskFileForSave(message='Save as:')
        if not path: return
        self.path = path
        self.name = os.path.split(self.path)[-1]
        self.wid.SetWTitle(self.name)
        self.menu_save()

    def menu_cut(self):
        self.ted.WESelView()
        if hasattr(Scrap, 'ZeroScrap'):
            Scrap.ZeroScrap()
        else:
            Scrap.ClearCurrentScrap()
        self.ted.WECut()
        self.updatescrollbars()
        self.parent.updatemenubar()
        self.changed = 1

    def menu_copy(self):
        if hasattr(Scrap, 'ZeroScrap'):
            Scrap.ZeroScrap()
        else:
            Scrap.ClearCurrentScrap()
        self.ted.WECopy()
        self.updatescrollbars()
        self.parent.updatemenubar()

    def menu_paste(self):
        self.ted.WESelView()
        self.ted.WEPaste()
        self.updatescrollbars()
        self.parent.updatemenubar()
        self.changed = 1

    def menu_clear(self):
        self.ted.WESelView()
        self.ted.WEDelete()
        self.updatescrollbars()
        self.parent.updatemenubar()
        self.changed = 1

    def menu_undo(self):
        self.ted.WEUndo()
        self.updatescrollbars()
        self.parent.updatemenubar()

    def have_selection(self):
        start, stop = self.ted.WEGetSelection()
        return start < stop

    def can_paste(self):
        return self.ted.WECanPaste()

    def can_undo(self):
        which, redo = self.ted.WEGetUndoInfo()
        which = UNDOLABELS[which]
        if which == None: return None
        if redo:
            return "Redo "+which
        else:
            return "Undo "+which

class Wed(Application):
    def __init__(self):
        Application.__init__(self)
        self.num = 0
        self.active = None
        self.updatemenubar()

    def makeusermenus(self):
        self.filemenu = m = Menu(self.menubar, "File")
        self.newitem = MenuItem(m, "New window", "N", self.open)
        self.openitem = MenuItem(m, "Open...", "O", self.openfile)
        self.closeitem = MenuItem(m, "Close", "W", self.closewin)
        m.addseparator()
        self.saveitem = MenuItem(m, "Save", "S", self.save)
        self.saveasitem = MenuItem(m, "Save as...", "", self.saveas)
        m.addseparator()
        self.quititem = MenuItem(m, "Quit", "Q", self.quit)

        self.editmenu = m = Menu(self.menubar, "Edit")
        self.undoitem = MenuItem(m, "Undo", "Z", self.undo)
        self.cutitem = MenuItem(m, "Cut", "X", self.cut)
        self.copyitem = MenuItem(m, "Copy", "C", self.copy)
        self.pasteitem = MenuItem(m, "Paste", "V", self.paste)
        self.clearitem = MenuItem(m, "Clear", "", self.clear)

        # Groups of items enabled together:
        self.windowgroup = [self.closeitem, self.saveitem, self.saveasitem, self.editmenu]
        self.focusgroup = [self.cutitem, self.copyitem, self.clearitem]
        self.windowgroup_on = -1
        self.focusgroup_on = -1
        self.pastegroup_on = -1
        self.undo_label = "never"

    def updatemenubar(self):
        changed = 0
        on = (self.active <> None)
        if on <> self.windowgroup_on:
            for m in self.windowgroup:
                m.enable(on)
            self.windowgroup_on = on
            changed = 1
        if on:
            # only if we have an edit menu
            on = self.active.have_selection()
            if on <> self.focusgroup_on:
                for m in self.focusgroup:
                    m.enable(on)
                self.focusgroup_on = on
                changed = 1
            on = self.active.can_paste()
            if on <> self.pastegroup_on:
                self.pasteitem.enable(on)
                self.pastegroup_on = on
                changed = 1
            on = self.active.can_undo()
            if on <> self.undo_label:
                if on:
                    self.undoitem.enable(1)
                    self.undoitem.settext(on)
                    self.undo_label = on
                else:
                    self.undoitem.settext("Nothing to undo")
                    self.undoitem.enable(0)
                changed = 1
        if changed:
            DrawMenuBar()

    #
    # Apple menu
    #

    def do_about(self, id, item, window, event):
        EasyDialogs.Message("A simple single-font text editor based on WASTE")

    #
    # File menu
    #

    def open(self, *args):
        self._open(0)

    def openfile(self, *args):
        self._open(1)

    def _open(self, askfile):
        if askfile:
            path = EasyDialogs.AskFileForOpen(typeList=('TEXT',))
            if not path:
                return
            name = os.path.split(path)[-1]
            try:
                fp = open(path, 'rb') # NOTE binary, we need cr as end-of-line
                data = fp.read()
                fp.close()
            except IOError, arg:
                EasyDialogs.Message("IOERROR: %r" % (arg,))
                return
        else:
            path = None
            name = "Untitled %d"%self.num
            data = ''
        w = WasteWindow(self)
        w.open(path, name, data)
        self.num = self.num + 1

    def closewin(self, *args):
        if self.active:
            self.active.close()
        else:
            EasyDialogs.Message("No active window?")

    def save(self, *args):
        if self.active:
            self.active.menu_save()
        else:
            EasyDialogs.Message("No active window?")

    def saveas(self, *args):
        if self.active:
            self.active.menu_save_as()
        else:
            EasyDialogs.Message("No active window?")


    def quit(self, *args):
        for w in self._windows.values():
            w.close()
        if self._windows:
            return
        self._quit()

    #
    # Edit menu
    #

    def undo(self, *args):
        if self.active:
            self.active.menu_undo()
        else:
            EasyDialogs.Message("No active window?")

    def cut(self, *args):
        if self.active:
            self.active.menu_cut()
        else:
            EasyDialogs.Message("No active window?")

    def copy(self, *args):
        if self.active:
            self.active.menu_copy()
        else:
            EasyDialogs.Message("No active window?")

    def paste(self, *args):
        if self.active:
            self.active.menu_paste()
        else:
            EasyDialogs.Message("No active window?")

    def clear(self, *args):
        if self.active:
            self.active.menu_clear()
        else:
            EasyDialogs.Message("No active window?")

    #
    # Other stuff
    #

    def idle(self, event):
        if self.active:
            self.active.do_idle(event)
        else:
            Qd.SetCursor(Qd.GetQDGlobalsArrow())

def main():
    App = Wed()
    App.mainloop()

if __name__ == '__main__':
    main()



from test.test_support import verbose, TestSkipped
import locale
import sys

if sys.platform == 'darwin':
    raise TestSkipped("Locale support on MacOSX is minimal and cannot be tested")
oldlocale = locale.setlocale(locale.LC_NUMERIC)

if sys.platform.startswith("win"):
    tloc = "en"
elif sys.platform.startswith("freebsd"):
    tloc = "en_US.US-ASCII"
else:
    tloc = "en_US"

try:
    locale.setlocale(locale.LC_NUMERIC, tloc)
except locale.Error:
    raise ImportError, "test locale %s not supported" % tloc

def testformat(formatstr, value, grouping = 0, output=None):
    if verbose:
        if output:
            print "%s %% %s =? %s ..." %\
                (repr(formatstr), repr(value), repr(output)),
        else:
            print "%s %% %s works? ..." % (repr(formatstr), repr(value)),
    result = locale.format(formatstr, value, grouping = grouping)
    if output and result != output:
        if verbose:
            print 'no'
        print "%s %% %s == %s != %s" %\
              (repr(formatstr), repr(value), repr(result), repr(output))
    else:
        if verbose:
            print "yes"

try:
    testformat("%f", 1024, grouping=1, output='1,024.000000')
    testformat("%f", 102, grouping=1, output='102.000000')
    testformat("%f", -42, grouping=1, output='-42.000000')
    testformat("%+f", -42, grouping=1, output='-42.000000')
    testformat("%20.f", -42, grouping=1, output='                 -42')
    testformat("%+10.f", -4200, grouping=1, output='    -4,200')
    testformat("%-10.f", 4200, grouping=1, output='4,200     ')
    # Invoke getpreferredencoding to make sure it does not cause exceptions,
    locale.getpreferredencoding()
finally:
    locale.setlocale(locale.LC_NUMERIC, oldlocale)


# Test BSD Rune locale's bug for isctype functions.
def teststrop(s, method, output):
    if verbose:
        print "%s.%s() =? %s ..." % (repr(s), method, repr(output)),
    result = getattr(s, method)()
    if result != output:
        if verbose:
            print "no"
        print "%s.%s() == %s != %s" % (repr(s), method, repr(result),
                                       repr(output))
    elif verbose:
        print "yes"

try:
    oldlocale = locale.setlocale(locale.LC_CTYPE)
    locale.setlocale(locale.LC_CTYPE, 'en_US.UTF-8')
except locale.Error:
    pass
else:
    try:
        teststrop('\x20', 'isspace', True)
        teststrop('\xa0', 'isspace', False)
        teststrop('\xa1', 'isspace', False)
        teststrop('\xc0', 'isalpha', False)
        teststrop('\xc0', 'isalnum', False)
        teststrop('\xc0', 'isupper', False)
        teststrop('\xc0', 'islower', False)
        teststrop('\xec\xa0\xbc', 'split', ['\xec\xa0\xbc'])
        teststrop('\xed\x95\xa0', 'strip', '\xed\x95\xa0')
        teststrop('\xcc\x85', 'lower', '\xcc\x85')
        teststrop('\xed\x95\xa0', 'upper', '\xed\x95\xa0')
    finally:
        locale.setlocale(locale.LC_CTYPE, oldlocale)



# This is a temporary setup script to allow distribution of
# MacPython 2.4 modules for MacPython 2.3.

from distutils.core import Extension, setup

setup(name="OSA", version="0.1",
        ext_modules=[
                Extension('_OSA', ['_OSAmodule.c'],
                extra_link_args=['-framework', 'Carbon'])
        ],
        py_modules=['OSA.OSA', 'OSA.OSAconst'],
        package_dir={'OSA':'../../../Lib/plat-mac/Carbon'}
        )



"""Unittests for heapq."""

from heapq import heappush, heappop, heapify, heapreplace, nlargest, nsmallest
import random
import unittest
from test import test_support
import sys


def heapiter(heap):
    # An iterator returning a heap's elements, smallest-first.
    try:
        while 1:
            yield heappop(heap)
    except IndexError:
        pass

class TestHeap(unittest.TestCase):

    def test_push_pop(self):
        # 1) Push 256 random numbers and pop them off, verifying all's OK.
        heap = []
        data = []
        self.check_invariant(heap)
        for i in range(256):
            item = random.random()
            data.append(item)
            heappush(heap, item)
            self.check_invariant(heap)
        results = []
        while heap:
            item = heappop(heap)
            self.check_invariant(heap)
            results.append(item)
        data_sorted = data[:]
        data_sorted.sort()
        self.assertEqual(data_sorted, results)
        # 2) Check that the invariant holds for a sorted array
        self.check_invariant(results)

        self.assertRaises(TypeError, heappush, [])
        self.assertRaises(TypeError, heappush, None, None)
        self.assertRaises(TypeError, heappop, None)

    def check_invariant(self, heap):
        # Check the heap invariant.
        for pos, item in enumerate(heap):
            if pos: # pos 0 has no parent
                parentpos = (pos-1) >> 1
                self.assert_(heap[parentpos] <= item)

    def test_heapify(self):
        for size in range(30):
            heap = [random.random() for dummy in range(size)]
            heapify(heap)
            self.check_invariant(heap)

        self.assertRaises(TypeError, heapify, None)

    def test_naive_nbest(self):
        data = [random.randrange(2000) for i in range(1000)]
        heap = []
        for item in data:
            heappush(heap, item)
            if len(heap) > 10:
                heappop(heap)
        heap.sort()
        self.assertEqual(heap, sorted(data)[-10:])

    def test_nbest(self):
        # Less-naive "N-best" algorithm, much faster (if len(data) is big
        # enough <wink>) than sorting all of data.  However, if we had a max
        # heap instead of a min heap, it could go faster still via
        # heapify'ing all of data (linear time), then doing 10 heappops
        # (10 log-time steps).
        data = [random.randrange(2000) for i in range(1000)]
        heap = data[:10]
        heapify(heap)
        for item in data[10:]:
            if item > heap[0]:  # this gets rarer the longer we run
                heapreplace(heap, item)
        self.assertEqual(list(heapiter(heap)), sorted(data)[-10:])

        self.assertRaises(TypeError, heapreplace, None)
        self.assertRaises(TypeError, heapreplace, None, None)
        self.assertRaises(IndexError, heapreplace, [], None)

    def test_heapsort(self):
        # Exercise everything with repeated heapsort checks
        for trial in xrange(100):
            size = random.randrange(50)
            data = [random.randrange(25) for i in range(size)]
            if trial & 1:     # Half of the time, use heapify
                heap = data[:]
                heapify(heap)
            else:             # The rest of the time, use heappush
                heap = []
                for item in data:
                    heappush(heap, item)
            heap_sorted = [heappop(heap) for i in range(size)]
            self.assertEqual(heap_sorted, sorted(data))

    def test_nsmallest(self):
        data = [random.randrange(2000) for i in range(1000)]
        for n in (0, 1, 2, 10, 100, 400, 999, 1000, 1100):
            self.assertEqual(nsmallest(n, data), sorted(data)[:n])

    def test_largest(self):
        data = [random.randrange(2000) for i in range(1000)]
        for n in (0, 1, 2, 10, 100, 400, 999, 1000, 1100):
            self.assertEqual(nlargest(n, data), sorted(data, reverse=True)[:n])

def test_main(verbose=None):
    test_classes = [TestHeap]
    test_support.run_unittest(*test_classes)

    # verify reference counting
    if verbose and hasattr(sys, "gettotalrefcount"):
        import gc
        counts = [None] * 5
        for i in xrange(len(counts)):
            test_support.run_unittest(*test_classes)
            gc.collect()
            counts[i] = sys.gettotalrefcount()
        print counts

if __name__ == "__main__":
    test_main(verbose=True)
